<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\model\sistema\Viaje;
use App\model\sistema\Tareas;
use App\model\sistema\Oficina;
use App\model\sistema\TipoTransaccion;
use App\model\core\Common;
use Validator;
use DateTime;
use Illuminate\Support\Facades\DB;

class CalendarioController extends Controller {

	public function __construct() {
		$this->middleware('auth');
	}

    public function index() {

		$viajes = Viaje::all()->where('estado','!=','1')->where('deleted','0');

		$resultado = array();
		foreach ($viajes as $data) {
			$fecha_start = $data['fecha_hora_salida'];
			$array_fecha_start = explode(' ', $fecha_start);
			$fecha_end = date('Y-m-d H:i:s', strtotime('+1 day', strtotime($data['fecha_hora_retorno'])));
			$array_fecha_end = explode(' ', $fecha_end);
			$fecha_end = $array_fecha_end[0]."T".$array_fecha_end[1];
			if ($array_fecha_start[0] == $array_fecha_end[0]) {
				$fecha_end = "";
			}
			$resultado[] = array("id_viaje"=>$data['id'],"viaje"=>$data->destino['ciudad'],"fecha_start"=>$array_fecha_start[0],"fecha_end"=>$fecha_end);
		}
		$viajes = $resultado;


		$fecha_hora_actual = date('Y-m-d h-m-s');
		$viajesSelect = Viaje::where('fecha_hora_retorno', '>=', $fecha_hora_actual)->orderBy('fecha_hora_salida', 'asc')->get();
		$viajesArray = array();
		foreach ($viajesSelect as $viaje) {
			$fecha = date("d-m-Y", strtotime($viaje->fecha_hora_salida));
			$dataDestino = DB::table('destino')->where('id', $viaje->destino_id)->first();
			$destino = $dataDestino->ciudad.', '.$dataDestino->provincia.' ('.$fecha.')';
			$viajesArray[] = array('id'=>$viaje->id,'destino'=>$destino);
		}
		$viajesSelect = $viajesArray;

		/* $tareas = Tareas::all()->where('estado','!=','1')->where('deleted','0');

		$resultado = array();
		foreach ($tareas as $tarea) {
			$resultado[] = array("id"=>$tarea->id,"tarea"=>$tarea->tarea,"fecha"=>$tarea->fecha);
		}
		$tareas = $resultado; */

    	return view('/calendario', ['viajes'=>$viajes,'viajesSelect'=>$viajesSelect,'title' => 'CALENDARIO']);
    }
	public function loadDataTareas($id_viaje) {

		$viaje = Viaje::where('id',$id_viaje)->first();
		$nombre_viaje = $viaje->destino['ciudad'];

		$tareas = Tareas::where([['deleted','0'],['id_viaje',$id_viaje]])->orderBy('fecha','asc')->get();
		$resultado = array();
		foreach ($tareas as $tarea) {
			$fechaFormat = date('d/m/Y', strtotime($tarea->fecha));
			$resultado[] = array("id"=>$tarea->id,"tarea"=>$tarea->tarea,"fecha"=>$fechaFormat,"estado"=>$tarea->estado);
		}
		$tareas = $resultado;

		$dataArray = array('nombre_viaje'=>$nombre_viaje,'tareas'=>$resultado);

    	return $dataArray;
    }
	public function loadDataTarea($id_tarea) {

		$dataTarea = Tareas::where([['id',$id_tarea]])->first();

		$viaje = Viaje::where('id',$dataTarea->id_viaje)->first();
		if (isset($viaje->id)) {
			$nombre_viaje = $viaje->destino['ciudad'];
		} else {
			$nombre_viaje = '';
		}

		$fechaFormat = date('d/m/Y', strtotime($dataTarea->fecha));

		$dataArray = array('fecha'=>$fechaFormat,'tarea'=>$dataTarea->tarea,'nombre_viaje'=>$nombre_viaje);

    	return $dataArray;
    }
	public function store_tarea(Request $request) {
		$fecha_d_m_y = \DateTime::createFromFormat('d/m/Y', $request->fecha)->format('Y-m-d');
		Tareas::create(['id_viaje'=>$request->id_viaje,'tarea'=>$request->tarea,'fecha'=>$fecha_d_m_y,'id_user'=>auth()->id()]);
		return redirect('/sistema/calendario');
	}
	public function edit_tarea(Request $request) {
		$id = $request->id_tarea;
		$fecha_d_m_y = \DateTime::createFromFormat('d/m/Y', $request->fecha)->format('Y-m-d');
		Tareas::find($id)->update(['tarea'=>$request->tarea,'fecha'=>$fecha_d_m_y,'id_user'=>auth()->id()]);
	}
	public function delete_tarea(Request $request) {
		$id = $request->id_tarea;
		Tareas::find($id)->delete();
	}
}
