<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\model\sistema\Destino;
use App\model\core\Image;
use App\model\sistema\Viaje;
use Illuminate\Support\Facades\Storage;
use Validator;

class DestinosController extends Controller
{
	public function __construct(Request $request) {
        $this->middleware('auth');
    }

    public function index() {
    	return view('destinos', ['title' => 'DESTINOS']); 
    }

    public function getAll() {
    	return Destino::with('image')->get()->toJson();
    }

    function save(Request $request) {
        $rules = [
            'ciudad' => 'required',
            'provincia' => 'required',
        ];

        $validator = Validator::make($request->all(), $rules, []);
        if($validator->fails()) {
            return response()->json(['errors'=>$validator->errors()->all()]);
        } else {
            $destino = $request['id'] ? Destino::get()->find($request['id']) : new Destino;
            $destino->ciudad = $request['ciudad'];
            $destino->provincia = $request['provincia'];
            $destino->detalle = $request['detalle'];
            if ($request['link_youtube']) {
                $videoArrayString = explode('v=', $request['link_youtube']);
                $destino->youtube_id = $videoArrayString[1];
            }
            $destino->id_user = auth()->id();
            $destino->save();

            if ($request->hasFile('images')) {
                $this->saveImages($request->images, $destino);
            }
            if (array_key_exists('deletedImages', $request->all())) {
                $this->deleteImagesById($request['deletedImages']);
            }

            return $destino->load('image');
        }
    }

    private function saveImages($images, $destino) {
        $count = Image::where('destino_id', $destino->id)->get()->count();
        foreach ($images as $imageFile) {
            $count++;
            $newName = $destino->id . '_' . $count . '.' . $imageFile->getClientOriginalExtension();
            $folderPath = 'images/destinos/' . $destino->id;
            $imageFile->move(public_path($folderPath), $newName);

            $dimensions = getimagesize($folderPath . '/' . $newName);
            $image = new Image;
            $image->name = $newName;
            $image->size = filesize($folderPath . '/' . $newName);
            $image->path = $folderPath . '/' . $newName;
            $image->width = $dimensions[0];
            $image->height = $dimensions[1];
            $image->destino_id = $destino->id;
            $image->save();
            $destino->image[] = $image;
        }
    }

    private function deleteImagesById(array $imagesId) {
        foreach ($imagesId as $id) {
            $imageToDelete = Image::where('id', $id)->get()->first();
            if (file_exists($imageToDelete->path)) {
                unlink($imageToDelete->path);
            }
            $imageToDelete->delete();
        }
    }

    public function remove(Request $request) {
        $viajesActivos = Viaje::where([
            ['destino_id', '=', $request->destinoId],
            ['estado', '=', 2]
        ])
        ->get()
        ->all();
        if (count($viajesActivos) > 0) {
            return 0;
        } else {
            Image::where('destino_id', $request->destinoId)->delete();
            Destino::destroy($request->destinoId);
            return 1;
        }
    }

    function bufferImages(Request $request) {
        $rules = [
            'file.*' => 'nullable|image|mimes:jpeg,png,jpg,gif|max:10240'
        ];
        $validator = Validator::make($request->all(), $rules, []);
        if($validator->fails()) {
            return response()->json(['errors'=>$validator->errors()->all()]);
        } else {
            foreach ($request['file'] as $image) {
                return response('ok');
            }
        }
    }

    public function save_imagen_destino(Request $request) {
        $id = $request->query('id');

        $count = Image::where('destino_id', $id)->get()->count();
        $count++;

        $path = 'images/destinos/'.$id;
        $file = $request->file('new_img');
        $new_image_name = $id.'_'.$count.'.webp';
        $upload = $file->move(public_path($path), $new_image_name);
        if($upload){
            $image = new Image;
            $image->name = $new_image_name;
            $image->path = $path . '/' . $new_image_name;
            $image->destino_id = $id;
            $image->save();

            $response = array('path'=>$path.'/'.$new_image_name, 'id_image'=>$image->id);

            return response()->json(['status'=>1, 'msg'=>$response]);
        } else {
            return response()->json(['status'=>0, 'msg'=>'Un error ocurrió en la subida de la imagen, por favor inténtalo nuevamente.']);
        }
    }

    public function delete_image_destino(Request $request) {
        $id = $request->id_image;

        $image = Image::find($id);

        if ($image) {
            if (file_exists($image->path)) {
                unlink($image->path);
            }

            try {
                $image->delete();
                return 1;
            } catch (\Exception $e) {
                return 0;
            }
        } else {
            return 0;
        }        
    }
}
