<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;

use App\model\sistema\Viaje;
use App\model\sistema\ReservaInfo;
use App\model\core\Common;
use App\model\sistema\Caja;
use App\model\sistema\CuotasExcursiones;
use App\model\sistema\Excursiones;
use App\model\sistema\ReservaExcursiones;
use Validator;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;

class ExcursionesController extends Controller
{
	public function __construct() {
		$this->middleware('auth');
	}

	public function index($id, Request $request) {
		$idRecibo = $request->query('id');

		$viaje = Viaje::find($id);
        $fecha_d_m_y = date("d/m/Y", strtotime($viaje->fecha_hora_salida));

		$excursiones = DB::table('excursiones')->where('id_viaje',$id)->get();
		$cant_excursiones = count($excursiones);

		$reservas = ReservaInfo::where('reserva_info.viaje_id', $id)
								->join('pasajero', 'pasajero.id', '=', 'reserva_info.pasajero_id')
								->where('pasajero.coordinador', '!=', 1)
								->select('pasajero.id as pasajero_id','pasajero.nombre','pasajero.apellido','reserva_info.id as id_reserva_info','reserva_info.infoExtraExcursiones','reserva_info.editores_info_excursiones')
								->get()->all();

		$arrayTabla = array();
		foreach ($reservas as $reserva) {
			$reserva_excursiones = DB::table('reserva_excursiones')
									->join('excursiones', 'excursiones.id', '=', 'reserva_excursiones.id_excursion')
									->where([['excursiones.id_viaje',$id],['reserva_excursiones.id_pasajero',$reserva->pasajero_id]])
									->select('excursiones.id')
									->get();

			$tarifa = 0;
			foreach ($excursiones as $excursion) {
				foreach ($reserva_excursiones as $exc) {
					if ($exc->id == $excursion->id) {
						$tarifa += $excursion->costo;
					}
				}
			}

			$entregado = 0;
			$cuotas = DB::table('cuotas_excursiones')
						->where([['id_viaje',$id],['id_pasajero',$reserva->pasajero_id]])
						->join('caja2020', 'caja2020.id', '=', 'cuotas_excursiones.id_caja')
						->select('caja2020.monto as cuota', 'caja2020.tipo_transaccion_id', 'caja2020.monto_cambio')
						->get();
			foreach ($cuotas as $cuota) {
				$monto_cambio = '1';
				if (($cuota->tipo_transaccion_id == "5" || $cuota->tipo_transaccion_id == "8" || $cuota->tipo_transaccion_id == "13") && is_numeric($cuota->monto_cambio)) {
					$monto_cambio = $cuota->monto_cambio;
				}
				$entregado += $cuota->cuota * $monto_cambio;
			}


			$saldo = $tarifa - $entregado;

			$arrayTabla[] = array(
				'id_reserva_info'=>$reserva->id_reserva_info,
				'id_pasajero'=>$reserva->pasajero_id,
				'nombre'=>$reserva->nombre,
				'apellido'=>$reserva->apellido,
				'infoExtra'=>$reserva->infoExtraExcursiones,
				'editoresInfoExtra'=>$reserva->editores_info_excursiones,
				'reserva_excursiones'=>$reserva_excursiones,
				'tarifa'=>$tarifa,
				'entregado'=>$entregado,
				'saldo'=>$saldo
			);
		}

		return view('excursiones', ['idRecibo'=>$idRecibo,'viajeId'=>$id,'excursiones'=>$excursiones,'cant_excursiones'=>$cant_excursiones,'dataTabla'=>$arrayTabla,'title' => $viaje->destino->ciudad . ' (' . $fecha_d_m_y . ') > EXCURSIONES']);
	}

	public function new_excursion(Request $request) {

		$new = new Excursiones();
		$new->id_viaje = $request->id_viaje;
		$new->nombre = $request->nombre;
		$new->costo = $request->costo;
		$new->save();

		return back();
	}

	public function edit_delete_excursion(Request $request) {

		if ($request->submit_button == "edit") {
			DB::table('excursiones')->where('id',$request->id_excursion)->update(['nombre'=>$request->nombre,'costo'=>$request->costo]);
		}

		if ($request->submit_button == "delete") {
			DB::table('excursiones')->where('id',$request->id_excursion)->delete();
			DB::table('reserva_excursiones')->where('id_excursion',$request->id_excursion)->delete();
		}

		return back();
	}

	public function changeExcursion(Request $request) {
		if ($request->tipo == 1) {
			$new = new ReservaExcursiones();
			$new->id_excursion = $request->id_excursion;
			$new->id_pasajero = $request->id_pasajero;
			$new->save();
		}
		if ($request->tipo == 0) {
			DB::table('reserva_excursiones')->where([['id_excursion',$request->id_excursion],['id_pasajero',$request->id_pasajero]])->delete();
		}
	}

	public function saveInfoExtra(Request $request) {
		DB::table('reserva_info')->where('id',$request->id)->update(['infoExtraExcursiones'=>$request->infoExtra, 'editores_info_excursiones'=>$request->editoresInfoExtra]);
	}
	
    public function getExcel($id) {

		$dato_viaje = Viaje::find($id);

		$viaje = $dato_viaje->destino->ciudad;
        $viaje = str_replace(",", "", $viaje);
        $viaje_url = str_replace(" ", "-", $viaje);
		$fecha_viaje = date('d-m-Y', strtotime($dato_viaje->fecha_hora_salida));

		$sumatoria_tarifa = 0;
		$sumatoria_entregado = 0;
		$sumatoria_saldo = 0;

		$excursiones = DB::table('excursiones')->where('id_viaje',$id)->get();

		$reservas = ReservaInfo::where('reserva_info.viaje_id', $id)
								->join('pasajero', 'pasajero.id', '=', 'reserva_info.pasajero_id')
								->where('pasajero.coordinador', '!=', 1)
								->select('pasajero.id as pasajero_id','pasajero.nombre','pasajero.apellido')
								->orderBy('pasajero.apellido', 'asc')
								->get()->all();

		$filename ="Excursiones-".$viaje_url."-(".$fecha_viaje.").xls";
		header('Content-type: application/ms-excel');
		header('Content-Disposition: attachment; filename='.$filename);

		$stuff="EXCURSIONES\n".$viaje."\t(".$fecha_viaje.")\t\t\n";
		$stuff=$stuff."\n";
		$stuff=$stuff."\tPASAJERO";
		foreach ($excursiones as $excursion) {
			$stuff=$stuff."\t".$excursion->nombre." ($".number_format($excursion->costo, 2, '.', '').")";
		}
		$stuff=$stuff."\tTARIFA\tENTREGADO\tSALDO\n";

		$stuff=$stuff."\n";

		foreach ($reservas as $reserva) {
			$reserva_excursiones = DB::table('reserva_excursiones')
									->join('excursiones', 'excursiones.id', '=', 'reserva_excursiones.id_excursion')
									->where([['excursiones.id_viaje',$id],['reserva_excursiones.id_pasajero',$reserva->pasajero_id]])
									->select('excursiones.id')
									->get();

			$tarifa = 0;
			foreach ($excursiones as $excursion) {
				foreach ($reserva_excursiones as $exc) {
					if ($exc->id == $excursion->id) {
						$tarifa += $excursion->costo;
					}
				}
			}

			$entregado = 0;
			$cuotas = DB::table('cuotas_excursiones')
						->where([['id_viaje',$id],['id_pasajero',$reserva->pasajero_id]])
						->join('caja2020', 'caja2020.id', '=', 'cuotas_excursiones.id_caja')
						->select('caja2020.monto as cuota')
						->get();
			foreach ($cuotas as $cuota) {
				$entregado += $cuota->cuota;
			}

			$saldo = $tarifa - $entregado;
			$sumatoria_tarifa += $tarifa;
			$sumatoria_entregado += $entregado;
			$sumatoria_saldo += $saldo;

			$stuff=$stuff."\t".$reserva->apellido." ".$reserva->nombre;

			$hace_excursion = 0;
			foreach ($excursiones as $excursion) {
				$check = "";
				foreach ($reserva_excursiones as $exc) {
					if ($exc->id == $excursion->id) {
						$check = "SI";
						$hace_excursion = 1;
					}
				}
				$stuff=$stuff."\t".$check;
			}

			if ($hace_excursion) {
				$stuff=$stuff."\t$".number_format($tarifa, 2, '.', '')."\t$".number_format($entregado, 2, '.', '')."\t$".number_format($saldo, 2, '.', '')."\n";
			} else {
				$stuff=$stuff."\t";
				if ($entregado == 0) {
					$stuff=$stuff."\t\t\n";
				} else {
					$stuff=$stuff."\t$".number_format($entregado, 2, '.', '')."\t$".number_format($saldo, 2, '.', '')."\n";
				}
			}
		}

		$stuff=$stuff."\n\tTOTAL TARIFA\t$".number_format($sumatoria_tarifa, 2, '.', '')."\n\tTOTAL ENTREGADO\t$".number_format($sumatoria_entregado, 2, '.', '')."\n\tTOTAL SALDO\t$".number_format($sumatoria_saldo, 2, '.', '')."\n";

        echo $stuff;        
    }

	public function getDataPasajero($id_pasajero, $id_viaje, $saldo_excursion) {
		$cuotas = DB::table('cuotas_excursiones')
					->where([['cuotas_excursiones.id_pasajero',$id_pasajero],['cuotas_excursiones.id_viaje',$id_viaje]])
					->join('caja2020', 'caja2020.id', '=', 'cuotas_excursiones.id_caja')
					->join('tipo_transaccion', 'tipo_transaccion.id', '=', 'caja2020.tipo_transaccion_id')
					->select('tipo_transaccion.nombre as medio_pago','caja2020.monto as cuota','cuotas_excursiones.id','cuotas_excursiones.id_caja','caja2020.fecha as timestamp','caja2020.tipo_transaccion_id','caja2020.monto_cambio')
					->get();

		$dataSaldos = DB::table('saldos_a_favor')->where('pasajero_id',$id_pasajero)->get();
		$sumatoria = 0;
		foreach ($dataSaldos as $data) {
			$saldo = $data->saldo;
			$saldo_usado = $data->saldo_usado;
			if ($saldo_usado != "") {
				$total = $saldo - $saldo_usado;
			} else {
				$total = $saldo;
			}
			$sumatoria = $sumatoria + $total;
		}
		$monto = "";
		if ($saldo_excursion > 0 && $sumatoria > 0) {
			if ($sumatoria <= $saldo_excursion) {
				$monto = $sumatoria;
			} else {
				$monto = $saldo_excursion;
			}
		}

		$data = array('cuotas'=>$cuotas,'saldo_a_favor'=>$monto);

		return $data;
	}

	public function add_cuota_excursion(Request $request) {

		$fecha = date('Y-m-d');
		$monto = str_replace(',','.',(str_replace('.','',$request->monto)));

		$caja = new Caja();
		$caja->fecha = $fecha;
		$caja->concepto = "Pago excursión de ".$request->nombre_pasajero;
		$caja->monto = $monto;
		if ($request->medio_pago == '5' || $request->medio_pago == '8' || $request->medio_pago == '13') {
			$caja->monto_cambio = $request->monto_cambio;
		}
		$caja->tipo_transaccion_id = $request->medio_pago;
		$caja->oficina_id = $request->oficina;
		$caja->id_user = auth()->id();
		$caja->save();

		$new = new CuotasExcursiones();
		$new->id_viaje = $request->id_viaje;
		$new->id_pasajero = $request->id_pasajero;
		/* $new->cuota = $monto;
		$new->medio_de_pago = $request->medio_pago; */
		$new->id_caja = $caja->id;
		$new->save();

		if ($request->medio_pago == '10') {
			$dataSaldos = DB::table('saldos_a_favor')->where('pasajero_id',$request->id_pasajero)->get();
			$saldo_resta = "";
			$monto_ingresado = $monto;
			foreach ($dataSaldos as $datos) {
				if ($saldo_resta == "") {
					$saldo = $datos->saldo;
					$saldo_usado = $datos->saldo_usado;
					if ($saldo_usado != "") {
						$saldo_disponible = $saldo - $saldo_usado;
					} else {
						$saldo_disponible = $saldo;
					}
					if ($saldo_disponible > 0) {
						$saldo_resta = $saldo_disponible - $monto_ingresado;
					}
					$comentario = DB::table('saldos_a_favor')->where('id',$datos->id)->value('comentario');
					$viaje = Viaje::where('id', $request->id_viaje)
						->with('destino')
						->first();
					DB::table('saldos_a_favor')->where('id',$datos->id)->update(['comentario'=>$comentario .'. Uso $'. $monto . ' para pago de excursiones en ' . $viaje->destino->ciudad . ' (' . date('d/m/Y', strtotime($viaje->fecha_hora_salida)) . ').']);
					if (!empty($saldo_resta) or $saldo_resta == "0") {
						if ($saldo_resta >= 0) {
							if ($saldo_usado != "") {
								$saldo_usado = $saldo_usado + $monto_ingresado;
								DB::table('saldos_a_favor')->where('id',$datos->id)->update(['saldo_usado'=>$saldo_usado,'id_user'=>auth()->id()]);
							} else {
								DB::table('saldos_a_favor')->where('id',$datos->id)->update(['saldo_usado'=>$monto_ingresado,'id_user'=>auth()->id()]);
							}
						}
						if ($saldo_resta < 0) {
							if ($saldo_usado != "") {
								$saldo_usado = $saldo_usado + $saldo_disponible;
								DB::table('saldos_a_favor')->where('id',$datos->id)->update(['saldo_usado'=>$saldo_usado,'id_user'=>auth()->id()]);
							} else {
								DB::table('saldos_a_favor')->where('id',$datos->id)->update(['saldo_usado'=>$saldo_disponible,'id_user'=>auth()->id()]);
							}
						}
						$nuevo_viaje = $datos->nuevo_viaje;
						if ($nuevo_viaje == "") {
							DB::table('saldos_a_favor')->where('id',$datos->id)->update(['nuevo_viaje'=>$request->id_viaje,'id_user'=>auth()->id()]);
						} else {
							$nuevo_viaje = $nuevo_viaje.','.$request->id_viaje;
							DB::table('saldos_a_favor')->where('id',$datos->id)->update(['nuevo_viaje'=>$nuevo_viaje,'id_user'=>auth()->id()]);
						}
						if (!empty($datos->fechas_usado)) {
							$fechas_usado = $datos->fechas_usado.', '.$fecha;
							DB::table('saldos_a_favor')->where('id',$datos->id)->update(['fechas_usado'=>$fechas_usado,'id_user'=>auth()->id()]);
						} else {
							DB::table('saldos_a_favor')->where('id',$datos->id)->update(['fechas_usado'=>$fecha,'id_user'=>auth()->id()]);
						}
					}
				} else {
					if ($saldo_resta < 0) {
						$saldo = $datos->saldo;
						$saldo_usado = $datos->saldo_usado;
						$saldo_pendiente = $saldo_resta * (-1);
						$saldo_resta = "";
						if ($saldo_usado != "") {
							$saldo_disponible = $saldo - $saldo_usado;
						} else {
							$saldo_disponible = $saldo;
						}
						if ($saldo_disponible > 0) {
							$saldo_resta = $saldo_disponible - $saldo_pendiente;
						}
						if ($saldo_resta != "") {
							if ($saldo_resta >= 0) {
								if ($saldo_usado != "") {
									$saldo_usado = $saldo_usado + $saldo_pendiente;
									DB::table('saldos_a_favor')->where('id',$datos->id)->update(['saldo_usado'=>$saldo_usado,'id_user'=>auth()->id()]);
								} else {
									DB::table('saldos_a_favor')->where('id',$datos->id)->update(['saldo_usado'=>$saldo_pendiente,'id_user'=>auth()->id()]);
								}
							}
							if ($saldo_resta < 0) {
								if ($saldo_usado != "") {
									$saldo_usado = $saldo_usado + $saldo_disponible;
									DB::table('saldos_a_favor')->where('id',$datos->id)->update(['saldo_usado'=>$saldo_usado,'id_user'=>auth()->id()]);
								} else {
									DB::table('saldos_a_favor')->where('id',$datos->id)->update(['saldo_usado'=>$saldo_disponible,'id_user'=>auth()->id()]);
								}
							}
						}
						$nuevo_viaje = $datos->nuevo_viaje;
						if ($nuevo_viaje == "") {
							DB::table('saldos_a_favor')->where('id',$datos->id)->update(['nuevo_viaje'=>$request->id_viaje,'id_user'=>auth()->id()]);
						} else {
							$nuevo_viaje = $nuevo_viaje.','.$request->id_viaje;
							DB::table('saldos_a_favor')->where('id',$datos->id)->update(['nuevo_viaje'=>$nuevo_viaje,'id_user'=>auth()->id()]);
						}
						if (!empty($datos->fechas_usado)) {
							$fechas_usado = $datos->fechas_usado.', '.$fecha;
							DB::table('saldos_a_favor')->where('id',$datos->id)->update(['fechas_usado'=>$fechas_usado,'id_user'=>auth()->id()]);
						} else {
							DB::table('saldos_a_favor')->where('id',$datos->id)->update(['fechas_usado'=>$fecha,'id_user'=>auth()->id()]);
						}
					}
				}
			}
		}

		if ($request->verRecibo == 1) {
			return redirect('/sistema/excursiones/'.$request->id_viaje.'?id='.$caja->id);
		} else {
			return redirect('/sistema/excursiones/'.$request->id_viaje);
		}
	}
}