<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\model\sistema\Pasajero;
use App\model\sistema\Reserva;
use App\model\sistema\ReservaInfo;
use App\model\sistema\Viaje;
use App\model\sistema\Caja;
use App\model\sistema\Cuota;
use App\model\sistema\Destino;
use App\model\sistema\TipoTransaccion;
use Validator;
use Illuminate\Support\Facades\DB;

class PasajeroController extends Controller {

	public function __construct(){
        $this->middleware('auth');
	}

    public function index(Request $request){
        $idRecibo = $request->query('id');
        $idPasajero = $request->query('idPasajero');
        $oficinas = DB::table('oficina')->orderBy('nombre','asc')->get();
        $tipo_transaccion = DB::table('tipo_transaccion')->orderBy('id','asc')->get();
    	return view('pasajeros', ['idRecibo'=>$idRecibo,'idPasajero'=>$idPasajero,'oficinas'=>$oficinas,'tipo_transaccion'=>$tipo_transaccion, 'title' => 'PASAJEROS']);
    }

    public function getAll() {
        $pasajeros = Pasajero::where('estado', 1)->with('reservainfo.viaje.destino')->get();
    
        $dateActual = date('Y-m-d');
        $arrayPasajeros = array();
    
        foreach ($pasajeros as $pasajero) {
            $arrayViajes = array();
    
            foreach ($pasajero->reservainfo as $reservaInfo) {
                $dataViaje = $reservaInfo->viaje;
    
                if ($dataViaje->fecha_hora_retorno >= $dateActual) {
                    $cuotas = DB::table('cuota2020')
                        ->where('reserva_info_id', $reservaInfo->id)
                        ->join('caja2020', 'caja2020.id', '=', 'cuota2020.transaction_id')
                        ->select('caja2020.*')
                        ->get();
    
                    $sumatoriaMontoCuotas = $cuotas->sum('monto');
                    $precioViaje = $reservaInfo->precio_opcional ?? $dataViaje->precio;
                    $saldoRestante = $precioViaje - $sumatoriaMontoCuotas;
    
                    if ($saldoRestante > 0) {
                        $destinoViaje = $dataViaje->destino;
                        $destino = $destinoViaje->ciudad . ', ' . $destinoViaje->provincia . ' (' . date('d-m-Y', strtotime($dataViaje->fecha_hora_salida)) . ')';
                        $arrayViajes[] = array('id' => $dataViaje->id, 'destino' => $destino, 'saldoRestante' => $saldoRestante);
                    }
                }
            }
    
            $arrayPasajeros[] = array(
                'id' => $pasajero->id,
                'nombre' => $pasajero->nombre,
                'apellido' => $pasajero->apellido,
                'dni' => $pasajero->dni,
                'fecha_nacimiento' => $pasajero->fecha_nacimiento,
                'domicilio' => $pasajero->domicilio,
                'localidad' => $pasajero->localidad,
                'telefono' => $pasajero->telefono,
                'email' => $pasajero->email,
                'comentarios' => $pasajero->comentarios,
                'estado' => $pasajero->estado,
                'viajes' => $arrayViajes
            );
        }
    
        return $arrayPasajeros;
    }

    public function validateAndSave(Request $request){
    	$data = $request->pasajero;
        $rules = [
            'nombre' => 'required',
            'apellido' => 'required',
            'dni' => $this->getUniqueDNIRule($data),
            'fecha_nacimiento' => 'required|date_format:"d/m/Y"',
            'localidad' => 'required',
            'telefono' => 'required|numeric'
        ];
        $validator = Validator::make($data, $rules, []);
        if($validator->fails()) {
            return response()->json(['errors' => $validator->errors()->all()]);
        }
        else {
            $data['fecha_nacimiento'] = \DateTime::createFromFormat('d/m/Y', $data['fecha_nacimiento'])->format('Y-m-d');
            return $this->savePasajero($data);
        }
    }

    private function getUniqueDNIRule($pasajero) {
        return array_key_exists('id', $pasajero) ? 'required|numeric|unique:pasajero,dni,' . $pasajero['id'] : 'required|numeric|unique:pasajero,dni';
    }

    private function savePasajero($data) {
        $pasajero = array_key_exists('id', $data) ? Pasajero::find($data['id']) : new Pasajero;
        $pasajero->nombre = $data['nombre'];
        $pasajero->apellido = $data['apellido'];
        $pasajero->dni = $data['dni'];
        $pasajero->fecha_nacimiento = $data['fecha_nacimiento'];
        //$pasajero->domicilio = $data['domicilio'];
        $pasajero->localidad = $data['localidad'];
        $pasajero->telefono = $data['telefono'];
        $pasajero->email = $data['email'];
        $pasajero->comentarios = $data['comentarios'];
        $pasajero->id_user = auth()->id();
        $pasajero->save();
        return $pasajero;
    }

    public function delete(Request $request) {
        Pasajero::where('id', $request->id)->update(['estado' => 0,'id_user'=>auth()->id()]);
        return $request->id;
    }

    public function getPasajeroInfo($id) {
        $data = [];
        $data['pasajero'] = Pasajero::find($id);
        $data['reservas'] = ReservaInfo::where('pasajero_id', $id)
            ->with('reserva.user', 'viaje.destino', 'cuotas', 'cuotas.transaction', 'cuotas.transaction.tipoTransaccion')
            ->get()->all();
        $data['reservasSinSaldo'] = 0;
        foreach ($data['reservas'] as $reserva) {
            $reserva->montoTotal = $reserva->precio_opcional ? $reserva->precio_opcional : $reserva->viaje->precio;
            foreach ($reserva->cuotas as $cuota) {
                $reserva->montoTotal -= $cuota->transaction->monto;
            }
            if ($reserva->montoTotal == 0) {
                $data['reservasSinSaldo']++;
            }
        }
        return view('pasajero-info', [
            'title' => $data['pasajero']->apellido . ', ' . $data['pasajero']->nombre,
            'data' => $data
        ]);
    }

    public function getExcel($id, $tipo) {

        $dato_viaje = Viaje::find($id);

        $reservas = ReservaInfo::where('viaje_id', $id)
                    ->with('reserva.user', 'viaje.destino', 'cuotas', 'cuotas.transaction', 'cuotas.transaction.tipoTransaccion')
                    ->orderBy('nro_asiento', 'ASC')->get()->all();

        $viaje = $dato_viaje->destino->ciudad;
        $viaje = str_replace(",", "", $viaje);
        $viaje_url = str_replace(" ", "-", $viaje);
        $separo_fecha = explode(' ', $dato_viaje->fecha_hora_salida);
        $fecha_d_m_y = date("d-m-Y", strtotime($separo_fecha[0]));
        $fecha_viaje = $fecha_d_m_y;

        //TODOS LOS DATOS
        if ($tipo == "1") {
            $filename ="Datos-".$viaje_url."-(".$fecha_viaje.").xls";
            header('Content-type: application/ms-excel');
            header('Content-Disposition: attachment; filename='.$filename);

            $stuff="DATOS\n".$viaje."\t(".$fecha_viaje.")\t\t\n";
            $stuff=$stuff."\n";
            $stuff=$stuff."\tCÓDIGO\tVENDEDOR\tN° ASIENTO\tPASAJERO\tDNI\tNACIMIENTO\tLOCALIDAD\tTELÉFONO\tPUNTO ASCENSO\tTIPO HABITACIÓN\tTARIFA\tENTREGADO\tSALDO\n";

            $stuff=$stuff."\n";

            foreach ($reservas as $dato) {
                $nombre_vendedor = $dato->reserva->user->lastName." ".$dato->reserva->user->name;
                $nombre_pasajero = $dato->pasajero->apellido." ".$dato->pasajero->nombre;
                $fecha_nacimiento = date("d/m/Y", strtotime($dato->pasajero->fecha_nacimiento));
                
                $sube_en = $dato->sube_en;
                if (empty($dato->sube_en)) {
                    $sube_en = $dato->pasajero->localidad;
                }

                $tipo_habitacion = "-";
                if ($dato->reserva->tipo_habitacion_id != null) {
                    $nombre_tipo_habit = DB::table('tipo_habitacion')->where('id', $dato->reserva->tipo_habitacion_id)->value('nombre');
                    $tipo_habitacion = $nombre_tipo_habit;
                }

                $simbolo_moneda = "";
                if ($dato->viaje->moneda == 1) {
                    $simbolo_moneda = "$";
                }
                if ($dato->viaje->moneda == 5) {
                    $simbolo_moneda = "U\$S";
                }
                if ($dato->viaje->moneda == 8) {
                    $simbolo_moneda = "€";
                }
                $precio_viaje = number_format($dato->viaje->precio, 2, ' ', '.');
                $precio_total = $dato->viaje->precio;
                if ($dato->precio_opcional != null) {
                    $precio_viaje = number_format($dato->precio_opcional, 2, ' ', '.');
                    $precio_total = $dato->precio_opcional;
                }
                if ($dato->precio_opcional == '0') {
                    $precio_viaje = number_format(0, 2, ' ', '.');
                    $precio_total = 0;
                }
                $tarifa = $simbolo_moneda." ".$precio_viaje;

                $entregado = "";
                $monto = 0;
                $array = json_decode($dato->cuotas, true);
                $sumatoria_monto = 0;
                $tipo_moneda = "$";
                foreach ($array as $value) {
                    if ($value['transaction']['tipo_transaccion_id'] == 1) {
                        $tipo_moneda = "$";
                    }
                    if ($value['transaction']['tipo_transaccion_id'] == 5) {
                        $tipo_moneda = "U\$S";
                    }
                    if ($value['transaction']['tipo_transaccion_id'] == 8) {
                        $tipo_moneda = "€";
                    }
                    $sumatoria_monto = $sumatoria_monto + $value['transaction']['monto'];
                }
                $entregado = $tipo_moneda." ".number_format($sumatoria_monto, 2, ' ', '.');
                $monto = $sumatoria_monto;

                $saldo_calc = $precio_total - $monto;
                $saldo = $simbolo_moneda." ".number_format($saldo_calc, 2, ' ', '.');

                $stuff=$stuff."\t".$dato->reserva->id."\t".$nombre_vendedor."\t".$dato->nro_asiento."\t".$nombre_pasajero."\t".$dato->pasajero->dni."\t".$fecha_nacimiento."\t".$dato->pasajero->localidad."\t";
                $stuff=$stuff.$dato->pasajero->telefono."\t".$sube_en."\t".$tipo_habitacion."\t".$tarifa."\t".$entregado."\t".$saldo."\n";

            }
        }
        //ASCENSOS
        if ($tipo == "2") {
            $filename ="Ascensos-".$viaje_url."-(".$fecha_viaje.").xls";
            header('Content-type: application/ms-excel');
            header('Content-Disposition: attachment; filename='.$filename);

            $stuff="ASCENSOS\n".$viaje."\t(".$fecha_viaje.")\t\t\n";
            $stuff=$stuff."\n";
            $stuff=$stuff."\tN° ASIENTO\tPASAJERO\tTELÉFONO\tPUNTO ASCENSO\n";

            $stuff=$stuff."\n";

            foreach ($reservas as $dato) {
                $nombre_pasajero = $dato->pasajero->apellido." ".$dato->pasajero->nombre;
                
                $sube_en = $dato->sube_en;
                if (empty($dato->sube_en)) {
                    $sube_en = $dato->pasajero->localidad;
                }

                $stuff=$stuff."\t".$dato->nro_asiento."\t".$nombre_pasajero."\t".$dato->pasajero->telefono."\t".$sube_en."\n";

            }
        }
        //PASAJEROS
        if ($tipo == "3") {
            $filename ="Pasajeros-".$viaje_url."-(".$fecha_viaje.").xls";
            header('Content-type: application/ms-excel');
            header('Content-Disposition: attachment; filename='.$filename);

            $stuff="PASAJEROS\n".$viaje."\t(".$fecha_viaje.")\t\t\n";
            $stuff=$stuff."\n";
            $stuff=$stuff."\tN° ASIENTO\tPASAJERO\tDNI\tFECHA DE NACIMIENTO\tTELÉFONO\n";

            $stuff=$stuff."\n";

            foreach ($reservas as $dato) {
                $nombre_pasajero = $dato->pasajero->apellido." ".$dato->pasajero->nombre;
                $fecha_nacimiento = date("d/m/Y", strtotime($dato->pasajero->fecha_nacimiento));

                $stuff=$stuff."\t".$dato->nro_asiento."\t".$nombre_pasajero."\t".$dato->pasajero->dni."\t".$fecha_nacimiento."\t".$dato->pasajero->telefono."\n";

            }
        }
        //ROOMING
        if ($tipo == "4") {
            $filename ="Rooming-".$viaje_url."-(".$fecha_viaje.").xls";
            header('Content-type: application/ms-excel');
            header('Content-Disposition: attachment; filename='.$filename);

            $stuff="ROOMING\n".$viaje."\t(".$fecha_viaje.")\t\t\n";
            $stuff=$stuff."\n";
            $stuff=$stuff."\tN° ASIENTO\tN° RESERVA\tPASAJERO\tTIPO DE HABITACIÓN\tINFO EXTRA\n";

            $stuff=$stuff."\n";

            foreach ($reservas as $dato) {
                $nombre_pasajero = $dato->pasajero->apellido." ".$dato->pasajero->nombre;
                
                $tipo_habitacion = "-";
                if ($dato->reserva->tipo_habitacion_id != null) {
                    $nombre_tipo_habit = DB::table('tipo_habitacion')->where('id', $dato->reserva->tipo_habitacion_id)->value('nombre');
                    $tipo_habitacion = $nombre_tipo_habit;
                }

                $stuff=$stuff."\t".$dato->nro_asiento."\t".$dato->reserva->id."\t".$nombre_pasajero."\t".$tipo_habitacion."\t".$dato->info."\n";

            }
        }
        //TRANSPORTE
        if ($tipo == "5") {
            $filename ="Transporte-".$viaje_url."-(".$fecha_viaje.").xls";
            header('Content-type: application/ms-excel');
            header('Content-Disposition: attachment; filename='.$filename);

            $stuff="TRANSPORTE\n".$viaje."\t(".$fecha_viaje.")\t\t\n";
            $stuff=$stuff."\n";
            $stuff=$stuff."\tN° ASIENTO\tPASAJERO\tDNI\tFECHA DE NACIMIENTO\n";

            $stuff=$stuff."\n";

            foreach ($reservas as $dato) {
                $nombre_pasajero = $dato->pasajero->apellido." ".$dato->pasajero->nombre;
                $fecha_nacimiento = date("d/m/Y", strtotime($dato->pasajero->fecha_nacimiento));

                $stuff=$stuff."\t".$dato->nro_asiento."\t".$nombre_pasajero."\t".$dato->pasajero->dni."\t".$fecha_nacimiento."\n";

            }
        }
        //EXCURSIONES
        if ($tipo == "6") {

            $viaje_incluye = DB::table('viaje_incluye')->where('viaje_id', $id)->get();

            $filename ="Excursiones-".$viaje_url."-(".$fecha_viaje.").xls";
            header('Content-type: application/ms-excel');
            header('Content-Disposition: attachment; filename='.$filename);

            $stuff="EXCURSIONES\n".$viaje."\t(".$fecha_viaje.")\t\t\n";
            $stuff=$stuff."\n";
            $stuff=$stuff."\tN° ASIENTO\tPASAJERO\tEXCURSIONES\t\tEXCURSIONES INCLUIDAS\tEXCURSIONES OPCIONALES\n";

            $stuff=$stuff."\n";

            $exc_inc = array();
            foreach ($viaje_incluye as $dato) {
                $excursion = substr($dato->descripcion, 0,15);

                if ($excursion == "Excursiones Inc" or $excursion == "Excursiones inc" or $excursion == "excursiones Inc" or $excursion == "excursiones inc") {
                    /* $stuff=$stuff."\n";
                    $stuff=$stuff."\tEXCURSIONES INCLUIDAS\n";
                    $stuff=$stuff."\n"; */

                    $array = explode(" - ", $dato->descripcion);

                    for ($i=1; $i < count($array); $i++) {
                        $exc = $array[$i];
                        $exc_inc[] = array("exc"=>$exc);
                        /* $stuff=$stuff."\t".$exc."\n"; */
                    }
                    /* $stuff=$stuff."\n"; */
                }
            }
            $exc_opc = array();
            foreach ($viaje_incluye as $dato) {
                $excursion = substr($dato->descripcion, 0,15);

                if ($excursion == "Excursiones Opc" or $excursion == "Excursiones opc" or $excursion == "excursiones Opc" or $excursion == "excursiones opc") {
                    /* $stuff=$stuff."\n";
                    $stuff=$stuff."\tEXCURSIONES OPCIONALES\n";
                    $stuff=$stuff."\n"; */

                    $array = explode(" - ", $dato->descripcion);

                    for ($i=1; $i < count($array); $i++) {
                        $exc = $array[$i];
                        $exc_opc[] = array("exc"=>$exc);
                        /* $stuff=$stuff."\t".$exc."\n"; */
                    }
                }
            }

            $count = 0;
            foreach ($reservas as $dato) {
                $nombre_pasajero = $dato->pasajero->apellido." ".$dato->pasajero->nombre;

                if (isset($exc_inc[$count]['exc'])) {
                    $dato_inc = $exc_inc[$count]['exc'];
                } else {
                    $dato_inc = "";
                }
                if (isset($exc_opc[$count]['exc'])) {
                    $dato_opc = $exc_opc[$count]['exc'];
                } else {
                    $dato_opc = "";
                }

                //return $exc_opc[5]['exc'];

                $stuff=$stuff."\t".$dato->nro_asiento."\t".$nombre_pasajero."\t\t\t".$dato_inc."\t".$dato_opc."\n";

                $count++;
            }

            $cant_max_exc = count($exc_inc);
            if (count($exc_inc) < count($exc_opc)) {
                $cant_max_exc = count($exc_opc);
            }

            if ($cant_max_exc > $count) {
                for ($i=0; $i < $cant_max_exc; $i++) {

                    if (isset($exc_inc[$i]['exc'])) {
                        $dato_inc = $exc_inc[$i]['exc'];
                    } else {
                        $dato_inc = "";
                    }
                    if (isset($exc_opc[$i]['exc'])) {
                        $dato_opc = $exc_opc[$i]['exc'];
                    } else {
                        $dato_opc = "";
                    }

                    $stuff=$stuff."\t\t\t\t\t".$dato_inc."\t".$dato_opc."\n";
                }
            }
        }

        echo $stuff;
        
    }

    public function saveCuota(Request $request) {
        $pasajero_id = $request->idPasajero;
        $dataPasajero = DB::table('pasajero')->where('id', $pasajero_id)->first();
        $viaje_id = $request->viajesAnotados;
        $data_reserva_info = DB::table('reserva_info')->where([['pasajero_id', $pasajero_id],['viaje_id', $viaje_id]])->first();
        $reserva_id = $data_reserva_info->reserva_id;
        $reserva_info_id = $data_reserva_info->id;

        $monto = $request->monto;
        $monto = str_replace(".", "", $monto);
        $monto = str_replace(",", ".", $monto);
        $tipo_transaccion_id = $request->medio_pago;
        $monto_cambio = $request->tipoCambio;
        $oficina_id = $request->oficina;

    	$transaction = new Caja;
		$transaction->fecha = date('Y-m-d');
		$transaction->concepto = 'Pago reserva n° ' . $reserva_id . ' ' . $dataPasajero->nombre . ' ' . $dataPasajero->apellido;
		$transaction->monto = $monto;
		$transaction->tipo_transaccion_id = $tipo_transaccion_id;
        if ($tipo_transaccion_id == 5 or  $tipo_transaccion_id == 8) {
            $transaction->monto_cambio = $monto_cambio;
        }
		$transaction->viaje_id = $viaje_id;
		$transaction->oficina_id = $oficina_id;
        $transaction->id_user = auth()->id();
		$transaction->save();

		$cuota = new Cuota;
    	$cuota->reserva_info_id = $reserva_info_id;
    	$cuota->transaction_id = $transaction->id;
    	$cuota->save();

        if ($request->verRecibo == 1) {
			return redirect('/sistema/pasajeros?id='.$transaction->id.'&idPasajero='.$pasajero_id);
		} else {
			return redirect('/sistema/pasajeros');
		}
    }

    public function getDataPasajero($id_pasajero) {
        $pasajero = Pasajero::where('id', $id_pasajero)->first();
        $destinos = Destino::get()->all();
        $tipos_transacciones = TipoTransaccion::get()->all();

        $viajes = DB::table('reserva_info')
                    ->where('pasajero_id', $id_pasajero)
                    ->join('viaje', 'viaje.id', '=', 'reserva_info.viaje_id')
                    ->where('viaje.fecha_hora_retorno', '>', date('Y-m-d 23:59:59'))
                    ->select('reserva_info.id','reserva_info.viaje_id','reserva_info.precio_opcional','viaje.destino_id','viaje.precio','viaje.fecha_hora_salida','viaje.fecha_hora_retorno')
                    ->get();

        $arrayViajes = array();
        $arrayCuotas = array();
        foreach ($viajes as $viaje) {
            $nombre_viaje = "";
            foreach ($destinos as $destino) {
                $fecha_viaje = date('d/m/Y', strtotime($viaje->fecha_hora_salida));
                if ($destino->id == $viaje->destino_id) {
                    $nombre_viaje = $destino->ciudad.' ('.$fecha_viaje.')';
                }
            }

            $cuotas = DB::table('cuota2020')
                            ->where([['reserva_info_id', $viaje->id]])
                            ->join('caja2020', 'caja2020.id', '=', 'cuota2020.transaction_id')
                            ->select('caja2020.*')
                            ->get();

            $arrayCuotas = array();
            $sumatoriaMontoCuotas = 0;
            foreach ($cuotas as $cuota) {
                $fechaCuota = date('d/m/Y', strtotime($cuota->fecha));
                foreach ($tipos_transacciones as $transac) {
                    if ($transac->id == $cuota->tipo_transaccion_id) {
                        $tipo_transaccion = $transac->nombre;
                    }
                }
                $monto = number_format($cuota->monto, 2, '.', ',');
                $arrayCuotas[] = array("fecha"=>$fechaCuota,"monto"=>$monto,"medio_pago"=>$tipo_transaccion,"id_caja"=>$cuota->id,);
                $sumatoriaMontoCuotas = $sumatoriaMontoCuotas + $cuota->monto;
            }

            $precioViaje = $viaje->precio;
            if (!empty($viaje->precio_opcional)) {
                $precioViaje = $viaje->precio_opcional;
            }
            $saldoRestante = $precioViaje - $sumatoriaMontoCuotas;
            $precioViaje = number_format($precioViaje, 2, '.', ',');
            $saldoRestante = number_format($saldoRestante, 2, '.', ',');

            if ($saldoRestante > 0) {
                $arrayViajes[] = array("id_viaje"=>$viaje->viaje_id,"viaje"=>$nombre_viaje,"fecha_viaje"=>$fecha_viaje,'precioViaje'=>$precioViaje,'saldoRestante'=>$saldoRestante,"cuotas"=>$arrayCuotas);
            }
        }
        $arrayViajes[] = array("id_viaje"=>0,"nombre"=>$pasajero->nombre,"apellido"=>$pasajero->apellido);

        return $arrayViajes;
    }
}
