<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\model\sistema\Viaje;
use App\model\sistema\Reserva;
use App\model\sistema\ReservaInfo;
use App\model\sistema\Locked;
use App\model\sistema\Pasajero;
use App\model\sistema\Users;
use App\model\sistema\RoomTypeRelation;
use App\model\sistema\Cuota;
use App\model\sistema\Caja;
use App\model\sistema\TipoTransaccion;
use App\model\core\Common;
use Validator;
use Auth;
use Illuminate\Support\Facades\DB;

class ReservaController extends Controller {

	public function __construct() {
        $this->middleware('auth');
	}

    public function index($viajeId) {
        $viaje = Viaje::find($viajeId);

        $separo_fecha = explode(' ', $viaje->fecha_hora_salida);
        $fecha_d_m_y = date("d/m/Y", strtotime($separo_fecha[0]));
        return view('reservar', ['viaje' => $viaje, 'title' => $viaje->destino->ciudad . ' (' . $fecha_d_m_y . ') > RESERVAR']);
    }

    public function getSeatsOccupied($viajeId) {

        //return ReservaInfo::where('viaje_id', $viajeId)->with('pasajero')->get()->keyBy('nro_asiento')->all();

        $ocupados = ReservaInfo::where('viaje_id', $viajeId)->with('pasajero')->get()->keyBy('nro_asiento')->all();
        $bloqueados = Locked::where('viaje_id', $viajeId)->get()->keyBy('butaca')->all();

        $dataArray = array("ocupados" => $ocupados, "bloqueados" => $bloqueados);

        return json_encode($dataArray);
    }

    private function getReservas($viajeId) {
        return Reserva::where('viaje_id', $viajeId)->get()->all();
    }

    private function getReservaInfo($viajeId) {
        return ReservaInfo::where('viaje_id', $viajeId)->get();
    }

    private function getHabitaciones($viajeId) {
        return RoomTypeRelation::where('viaje_id', $viajeId)->get();
    }

    public function getFormData($viajeId, Request $request) {
        $data = [];
        $pasajerosIds = ReservaInfo::where('viaje_id', $viajeId)
            ->select('pasajero_id')
            ->get();
        $data['pasajeros'] = Pasajero::whereNotIn('id', $pasajerosIds)
            ->get();
        $data['roomTypes'] = json_encode(RoomController::getForViaje($viajeId), JSON_NUMERIC_CHECK);
        $data['reservas'] = json_encode($this->getReservas($viajeId), JSON_NUMERIC_CHECK);
        $data['reservainfo'] = json_encode($this->getReservaInfo($viajeId), JSON_NUMERIC_CHECK);
        //$data['habitaciones'] = json_encode($this->getHabitaciones($viajeId), JSON_NUMERIC_CHECK);

        if (boolval($request->isUserAdmin)) {
            $users = Users::orderBy('lastname')->get()->all();
            foreach ($users as $user) { unset($user->password); }
            $data['users'] = $users;
        }
        return $data;
    }

    public function validar($id, Request $request) {

        $dataPasajero = DB::table('pasajero')->where('id',$request['selectedSeats'][0]['pasajero_id'])->first();

        $firstRules = [];
        if ($request['solo_transporte'] != '1') {
            if ($request['cantNoches'] > 0 && boolval($request['areRoomsAvailable'] && $dataPasajero->coordinador != "1")) {
                $firstRules ['roomSelected'] = 'required';
            }
        }
        if ($this->isUserLoggedAdmin()) {
            $firstRules['user_id'] = 'required';
        }
        $messages = $this->getReservaValidatorMessages();
        $validator = Validator::make($request->all(), $firstRules, $messages);
        if($validator->fails()) {
            return response()->json(['errors' => $validator->errors()->all()]);
        }
        $rules = ['pasajero_id' => 'required'];
        $data = [];
        $data['messages'] = [];
        $i = 0;
        foreach ($request['selectedSeats'] as $reserva) {
            $validator = Validator::make($reserva, $rules, $messages);
            if($validator->fails()) {
                $data['messages'][$i] = new \stdClass;
                $data['messages'][$i]->key = $reserva['nro_asiento'];
                $data['messages'][$i++]->value = response()->json(['errors'=>$validator->errors()->all()]);
            }
        }
        return $data;
    }

    private function getReservaValidatorMessages() {
        return [
            'roomSelected.required' => 'Debe seleccionar un tipo de habitaci&oacute;n.',
            'user_id.required' => 'Debe seleccionar el usuario que concret&oacute; esta reserva.',
            'pasajero_id.required' => 'Debe seleccionar un pasajero o crear uno nuevo.'
        ];
    }

    public function confirmar($viajeId, Request $request) {
        $reserva = new Reserva;
        $reserva->viaje_id = $viajeId;
        $reserva->user_id = $this->isUserLoggedAdmin() ? (int) $request['user_id'] : Auth::id();
        $reserva->tipo_habitacion_id = $request['tipo_habitacion_id'];
        if ($request['tipo_habitacion_id'] == 4 || $request['tipo_habitacion_id'] == 5) {
            $reserva->twin_matrimonial = $request['tipo_hab'];
        }
        $reserva->created_at = date('Y-m-d H:i:s');
        $reserva->user_id_reservo = Auth::id();
        $reserva->save();
        foreach ($request['selectedSeats'] as $seat) {
            $info = new ReservaInfo;
            $info->reserva_id = $reserva->id;
            $info->viaje_id = $viajeId;
            $info->pasajero_id = $seat['pasajero_id'];
            $info->nro_asiento = $seat['nro_asiento'];
            $info->precio_opcional = $seat['precio_opcional'];
            $info->solo_transporte = $request['solo_transporte'];
            $info->save();
        }
        return $reserva;
    }

    public function confirmacion($id, Request $request) {
        $viaje = Viaje::where('id', $id)
            ->with('destino')
            ->first();
        $selectedSeats = json_decode($request['seats']);
        $roomSelected = RoomController::find($request['roomSelected']);
        $hayPasajero = 0;
        foreach ($selectedSeats as $seat) {
            if (isset($seat->pasajero->coordinador)) {
                if ($seat->pasajero->coordinador != 1) {
                    $hayPasajero = 1;
                }
            } else {
                $hayPasajero = 1;
            }
        }
        if ($hayPasajero == 0) {
            return redirect('/sistema/cuentas/'.$id);
        } else {
            return view('reservaConfirm', [
                'selectedSeats' => $selectedSeats,
                'roomSelected' => $roomSelected,
                'viaje' => $viaje,
                'title' => $viaje->destino->ciudad . ' > CONFIRMACIÓN DE RESERVA',
                'reserva' => json_decode($request['reserva'])
            ]);
        }
    }

    public function getDataIdReservaInfo($reserva_id, $pasajero_id, $viaje_id) {
        $id = ReservaInfo::where([['reserva_id','=',$reserva_id],['pasajero_id','=',$pasajero_id],['viaje_id','=',$viaje_id]])->value('id');
        $cuotas = Cuota::where([['reserva_info_id',$id]])->get();

        /* $resultado = array();

        //$cuotas_array = '[';
        foreach ($cuotas as $cuota) {
            //$cuotas_array = $cuotas_array.'{"id": '.$cuota->id.',"reserva_info_id": "'.$cuota->reserva_info_id.'","transaction_id": "'.$cuota->transaction_id.'",';
            $datos_transaccion = Caja::where('id',$cuota->transaction_id)->first();
            //$cuotas_array = $cuotas_array.'"transaction": {"id": '.$datos_transaccion->id.',"nro_factura": "'.$datos_transaccion->nro_factura.'","fecha": "'.$datos_transaccion->fecha.'","concepto": "'.$datos_transaccion->concepto.'","monto": "'.$datos_transaccion->monto.'","tipo_transaccion_id": "'.$datos_transaccion->tipo_transaccion_id.'","metodo_pago": "'.$datos_transaccion->metodo_pago.'","monto_cambio": "'.$datos_transaccion->monto_cambio.'","viaje_id": "'.$datos_transaccion->viaje_id.'","oficina_id": "'.$datos_transaccion->oficina_id.'",';
            $datos_tipo_transaccion = TipoTransaccion::where('id',$datos_transaccion->tipo_transaccion_id)->first();
            //$cuotas_array = $cuotas_array.'"tipo_transaccion": {"id": '.$datos_tipo_transaccion->id.',"nombre": "'.$datos_tipo_transaccion->nombre.'"}';
            //$cuotas_array = $cuotas_array.'}';
            //$cuotas_array = $cuotas_array.'};';

            $transaction_data = array("id"=>$datos_transaccion->id,"nro_factura"=>$datos_transaccion->nro_factura,"fecha"=>$datos_transaccion->fecha,"concepto"=>$datos_transaccion->concepto,"monto"=>$datos_transaccion->monto,"tipo_transaccion_id"=>$datos_transaccion->tipo_transaccion_id,"metodo_pago"=>$datos_transaccion->metodo_pago,"monto_cambio"=>$datos_transaccion->monto_cambio,"viaje_id"=>$datos_transaccion->viaje_id,"oficina_id"=>$datos_transaccion->oficina_id);
            $tipo_transaccion_data = array("id"=>$datos_tipo_transaccion->id,"nombre"=>$datos_tipo_transaccion->nombre);

            $resultado[] = array("id"=>$cuota->id,"reserva_info_id"=>$cuota->reserva_info_id,"transaction_id"=>$cuota->transaction_id,"transaction"=>$transaction_data,"tipo_transaccion"=>$tipo_transaccion_data);

        }
        //$cuotas_array = $cuotas_array.']';

        $cuotas_array = json_encode($resultado);

        //$cuotas_array = json_encode($cuotas_array);

        //echo $cuotas_array;

        //$response = '{"id": '.$id.',"cuotas": '.$cuotas_array.'}';
        $response = $id.',_,_,'.$cuotas_array;

        //$response[] = array("id"=>$id,"reserva_id"=>$reserva_id,"pasajero_id"=>$pasajero_id,"viaje_id"=>$viaje_id,"cuotas"=>$resultado);
        //$response = json_encode($response);

        return $response; */

        $response = array();
        $cuotas_array = array();
        foreach ($cuotas as $cuota) {
            
            $datos_transaccion = Caja::where('id',$cuota->transaction_id)->first();
            
            $datos_tipo_transaccion = TipoTransaccion::where('id',$datos_transaccion->tipo_transaccion_id)->first();

            $tipo_transaccion_data = array("id"=>$datos_tipo_transaccion->id,"nombre"=>$datos_tipo_transaccion->nombre);
            
            $transaction_data = array("id"=>$datos_transaccion->id,"nro_factura"=>$datos_transaccion->nro_factura,"fecha"=>$datos_transaccion->fecha,"concepto"=>$datos_transaccion->concepto,"monto"=>$datos_transaccion->monto,"tipo_transaccion_id"=>$datos_transaccion->tipo_transaccion_id,"metodo_pago"=>$datos_transaccion->metodo_pago,"monto_cambio"=>$datos_transaccion->monto_cambio,"viaje_id"=>$datos_transaccion->viaje_id,"oficina_id"=>$datos_transaccion->oficina_id,"tipo_transaccion"=>$tipo_transaccion_data);
            
            $cuotas_array[] = array("id"=>$cuota->id,"reserva_info_id"=>$cuota->reserva_info_id,"transaction_id"=>$cuota->transaction_id,"transaction"=>$transaction_data,"tipo_transaccion"=>$tipo_transaccion_data);

        }
        $response = array("id"=>$id,"reserva_id"=>$reserva_id,"pasajero_id"=>$pasajero_id,"viaje_id"=>$viaje_id,"cuotas"=>$cuotas_array);

        //$response = json_encode($response);

        return $response;

    }

    private function isUserLoggedAdmin() {
        return Auth::user()->role == 2;
    }

    public function blockSeats($id, $tipo, $seat) {
        $explode_seat = explode('-', $seat);

        foreach ($explode_seat as $seats) {
            $new_locked = new Locked;
            $new_locked->viaje_id = $id;
            $new_locked->butaca = $seats;
            $new_locked->tipo_bloqueo = $tipo;
            $new_locked->created_at = date('Y-m-d H:i:s');
            $new_locked->id_user = auth()->id();
            $new_locked->save();
        }
    }

    public function enableSeats($id, $seat) {
        Locked::where('viaje_id',$id)->where('butaca',$seat)->delete();
    }
}
