<?php

namespace App\Http\Controllers;

use App\model\sistema\Caja;
use App\model\sistema\CuotasSeguros;
use App\model\sistema\Seguros;
use Illuminate\Http\Request;

use App\model\sistema\Viaje;
use App\model\sistema\ReservaInfo;
use Illuminate\Support\Facades\DB;

class SegurosController extends Controller
{
	public function __construct() {
		$this->middleware('auth');
	}

	public function index($id, Request $request) {
		$idRecibo = $request->query('id');

		$viaje = Viaje::find($id);
        $fecha_d_m_y = date("d/m/Y", strtotime($viaje->fecha_hora_salida));

		$reservas = ReservaInfo::where('reserva_info.viaje_id', $id)
								->join('pasajero', 'pasajero.id', '=', 'reserva_info.pasajero_id')
								->leftJoin('seguros', 'seguros.id_pasajero', '=', 'reserva_info.pasajero_id')
								->where('pasajero.coordinador', '!=', 1)
								->select(
									'pasajero.id as pasajero_id',
									'pasajero.nombre',
									'pasajero.apellido',
									'reserva_info.id as id_reserva_info',
									'seguros.ofrecido as ofrecido_seguro',
									'seguros.contratado as contratado_seguro',
									'seguros.costo as costo_seguro',
									'seguros.pago_prestadora',
									'seguros.id as id_seguro',
								)
								->get()->all();

		$arrayTabla = array();
		foreach ($reservas as $reserva) {

			$tarifa = $reserva->costo_seguro;

			$entregado = 0;
			$cuotas = DB::table('cuotas_seguros')
						->where([['id_viaje',$id],['id_pasajero',$reserva->pasajero_id]])
						->join('caja2020', 'caja2020.id', '=', 'cuotas_seguros.id_caja')
						->select('caja2020.monto as cuota', 'caja2020.tipo_transaccion_id', 'caja2020.monto_cambio')
						->get();
			
			foreach ($cuotas as $cuota) {
				$monto_cambio = '1';
				if (($cuota->tipo_transaccion_id == "5" || $cuota->tipo_transaccion_id == "8" || $cuota->tipo_transaccion_id == "13") && is_numeric($cuota->monto_cambio)) {
					$monto_cambio = $cuota->monto_cambio;
				}
				$entregado += $cuota->cuota * $monto_cambio;
			}

			$saldo = $tarifa - $entregado;

			$arrayTabla[] = array(
				'id_reserva_info'=>$reserva->id_reserva_info,
				'id_pasajero'=>$reserva->pasajero_id,
				'nombre'=>$reserva->nombre,
				'apellido'=>$reserva->apellido,
				'infoExtra'=>'',
				'editoresInfoExtra'=>'',
				'tarifa'=>$tarifa,
				'entregado'=>$entregado,
				'saldo'=>$saldo,
				'ofrecido'=>$reserva->ofrecido_seguro,
				'contratado'=>$reserva->contratado_seguro,
				'id_seguro'=>$reserva->id_seguro ?? '-1',
				'pago_prestadora'=>$reserva->pago_prestadora,
			);
		}

		//return $arrayTabla;

		return view('seguros', ['idRecibo'=>$idRecibo,'viajeId'=>$id,'dataTabla'=>$arrayTabla,'title' => $viaje->destino->ciudad . ' (' . $fecha_d_m_y . ') > SEGURO AL VIAJERO']);
	}

	public function new_seguro(Request $request) {

		$new = new Seguros();
		$new->id_viaje = $request->id_viaje;
		$new->costo = $request->costo;
		$new->save();

		return back();
	}

	public function getDataPasajero($id_pasajero, $id_viaje, $saldo_seguro) {
		$cuotas = DB::table('cuotas_seguros')
					->where([['cuotas_seguros.id_pasajero',$id_pasajero],['cuotas_seguros.id_viaje',$id_viaje]])
					->join('caja2020', 'caja2020.id', '=', 'cuotas_seguros.id_caja')
					->join('tipo_transaccion', 'tipo_transaccion.id', '=', 'caja2020.tipo_transaccion_id')
					->select('tipo_transaccion.nombre as medio_pago','caja2020.monto as cuota','cuotas_seguros.id','cuotas_seguros.id_caja','caja2020.fecha as timestamp','caja2020.tipo_transaccion_id','caja2020.monto_cambio')
					->get();

		$dataSaldos = DB::table('saldos_a_favor')->where('pasajero_id',$id_pasajero)->get();
		$sumatoria = 0;
		foreach ($dataSaldos as $data) {
			$saldo = $data->saldo;
			$saldo_usado = $data->saldo_usado;
			if ($saldo_usado != "") {
				$total = $saldo - $saldo_usado;
			} else {
				$total = $saldo;
			}
			$sumatoria = $sumatoria + $total;
		}
		$monto = "";
		if ($saldo_seguro > 0 && $sumatoria > 0) {
			if ($sumatoria <= $saldo_seguro) {
				$monto = $sumatoria;
			} else {
				$monto = $saldo_seguro;
			}
		}

		$data = array('cuotas'=>$cuotas,'saldo_a_favor'=>$monto);

		return $data;
	}

	public function add_cuota_seguro(Request $request) {

		$fecha = date('Y-m-d');
		$monto = str_replace(',','.',(str_replace('.','',$request->monto)));

		if ($request->id_seguro == '-1') {
			$caja = new Seguros();
			$caja->id_viaje = $request->id_viaje;
			$caja->id_pasajero = $request->id_pasajero;
			$caja->costo = str_replace(',','.',(str_replace('.','',$request->monto_seguro)));
			$caja->pago_prestadora = str_replace(',','.',(str_replace('.','',$request->pago_prestadora)));
			$caja->save();
		} else {
			$data_seguro = DB::table('seguros')->where('id', $request->id_seguro)->first();

			if ($data_seguro->costo != $request->costo) {
				DB::table('seguros')->where('id', $request->id_seguro)->update(['costo'=>$request->costo]);
			}
			if ($data_seguro->pago_prestadora != $request->pago_prestadora) {
				DB::table('seguros')->where('id', $request->id_seguro)->update(['pago_prestadora'=>$request->pago_prestadora]);

				$id_caja = DB::table('cuotas_seguros')->where([['id_viaje', $request->id_viaje], ['id_pasajero', $request->id_pasajero]])->value('id_caja');
				DB::table('caja2020')->where('id', $id_caja)->update(['monto'=>$request->pago_prestadora]);
			}
		}

		if (!empty($request->monto)) {
			$caja = new Caja();
			$caja->fecha = $fecha;
			$caja->concepto = "Pago seguro de ".$request->nombre_pasajero;
			$caja->monto = $monto;
			if ($request->medio_pago == '5' || $request->medio_pago == '8' || $request->medio_pago == '13') {
				$caja->monto_cambio = $request->monto_cambio;
			}
			$caja->tipo_transaccion_id = $request->medio_pago;
			$caja->oficina_id = $request->oficina;
			$caja->id_user = auth()->id();
			$caja->save();

			$new = new CuotasSeguros();
			$new->id_viaje = $request->id_viaje;
			$new->id_pasajero = $request->id_pasajero;
			$new->id_caja = $caja->id;
			$new->save();

			if ($request->medio_pago == '10') {
				$dataSaldos = DB::table('saldos_a_favor')->where('pasajero_id',$request->id_pasajero)->get();
				$saldo_resta = "";
				$monto_ingresado = $monto;
				foreach ($dataSaldos as $datos) {
					if ($saldo_resta == "") {
						$saldo = $datos->saldo;
						$saldo_usado = $datos->saldo_usado;
						if ($saldo_usado != "") {
							$saldo_disponible = $saldo - $saldo_usado;
						} else {
							$saldo_disponible = $saldo;
						}
						if ($saldo_disponible > 0) {
							$saldo_resta = $saldo_disponible - $monto_ingresado;
						}
						$comentario = DB::table('saldos_a_favor')->where('id',$datos->id)->value('comentario');
						$viaje = Viaje::where('id', $request->id_viaje)
							->with('destino')
							->first();
						DB::table('saldos_a_favor')->where('id',$datos->id)->update(['comentario'=>$comentario .'. Uso $'. $monto . ' para pago de seguro en ' . $viaje->destino->ciudad . ' (' . date('d/m/Y', strtotime($viaje->fecha_hora_salida)) . ').']);
						if (!empty($saldo_resta) or $saldo_resta == "0") {
							if ($saldo_resta >= 0) {
								if ($saldo_usado != "") {
									$saldo_usado = $saldo_usado + $monto_ingresado;
									DB::table('saldos_a_favor')->where('id',$datos->id)->update(['saldo_usado'=>$saldo_usado,'id_user'=>auth()->id()]);
								} else {
									DB::table('saldos_a_favor')->where('id',$datos->id)->update(['saldo_usado'=>$monto_ingresado,'id_user'=>auth()->id()]);
								}
							}
							if ($saldo_resta < 0) {
								if ($saldo_usado != "") {
									$saldo_usado = $saldo_usado + $saldo_disponible;
									DB::table('saldos_a_favor')->where('id',$datos->id)->update(['saldo_usado'=>$saldo_usado,'id_user'=>auth()->id()]);
								} else {
									DB::table('saldos_a_favor')->where('id',$datos->id)->update(['saldo_usado'=>$saldo_disponible,'id_user'=>auth()->id()]);
								}
							}
							$nuevo_viaje = $datos->nuevo_viaje;
							if ($nuevo_viaje == "") {
								DB::table('saldos_a_favor')->where('id',$datos->id)->update(['nuevo_viaje'=>$request->id_viaje,'id_user'=>auth()->id()]);
							} else {
								$nuevo_viaje = $nuevo_viaje.','.$request->id_viaje;
								DB::table('saldos_a_favor')->where('id',$datos->id)->update(['nuevo_viaje'=>$nuevo_viaje,'id_user'=>auth()->id()]);
							}
							if (!empty($datos->fechas_usado)) {
								$fechas_usado = $datos->fechas_usado.', '.$fecha;
								DB::table('saldos_a_favor')->where('id',$datos->id)->update(['fechas_usado'=>$fechas_usado,'id_user'=>auth()->id()]);
							} else {
								DB::table('saldos_a_favor')->where('id',$datos->id)->update(['fechas_usado'=>$fecha,'id_user'=>auth()->id()]);
							}
						}
					} else {
						if ($saldo_resta < 0) {
							$saldo = $datos->saldo;
							$saldo_usado = $datos->saldo_usado;
							$saldo_pendiente = $saldo_resta * (-1);
							$saldo_resta = "";
							if ($saldo_usado != "") {
								$saldo_disponible = $saldo - $saldo_usado;
							} else {
								$saldo_disponible = $saldo;
							}
							if ($saldo_disponible > 0) {
								$saldo_resta = $saldo_disponible - $saldo_pendiente;
							}
							if ($saldo_resta != "") {
								if ($saldo_resta >= 0) {
									if ($saldo_usado != "") {
										$saldo_usado = $saldo_usado + $saldo_pendiente;
										DB::table('saldos_a_favor')->where('id',$datos->id)->update(['saldo_usado'=>$saldo_usado,'id_user'=>auth()->id()]);
									} else {
										DB::table('saldos_a_favor')->where('id',$datos->id)->update(['saldo_usado'=>$saldo_pendiente,'id_user'=>auth()->id()]);
									}
								}
								if ($saldo_resta < 0) {
									if ($saldo_usado != "") {
										$saldo_usado = $saldo_usado + $saldo_disponible;
										DB::table('saldos_a_favor')->where('id',$datos->id)->update(['saldo_usado'=>$saldo_usado,'id_user'=>auth()->id()]);
									} else {
										DB::table('saldos_a_favor')->where('id',$datos->id)->update(['saldo_usado'=>$saldo_disponible,'id_user'=>auth()->id()]);
									}
								}
							}
							$nuevo_viaje = $datos->nuevo_viaje;
							if ($nuevo_viaje == "") {
								DB::table('saldos_a_favor')->where('id',$datos->id)->update(['nuevo_viaje'=>$request->id_viaje,'id_user'=>auth()->id()]);
							} else {
								$nuevo_viaje = $nuevo_viaje.','.$request->id_viaje;
								DB::table('saldos_a_favor')->where('id',$datos->id)->update(['nuevo_viaje'=>$nuevo_viaje,'id_user'=>auth()->id()]);
							}
							if (!empty($datos->fechas_usado)) {
								$fechas_usado = $datos->fechas_usado.', '.$fecha;
								DB::table('saldos_a_favor')->where('id',$datos->id)->update(['fechas_usado'=>$fechas_usado,'id_user'=>auth()->id()]);
							} else {
								DB::table('saldos_a_favor')->where('id',$datos->id)->update(['fechas_usado'=>$fecha,'id_user'=>auth()->id()]);
							}
						}
					}
				}
			}
		}

		if ($request->verRecibo == 1) {
			return redirect('/sistema/seguro-al-viajero/'.$request->id_viaje.'?id='.$caja->id);
		} else {
			return redirect('/sistema/seguro-al-viajero/'.$request->id_viaje);
		}
	}

	public function ofrecido_contratado(Request $request) {

		if ($request->id_seguro == '-1') {
			$contratado = 0;
			$ofrecido = 0;
			if ($request->tipo == "contratado") {
				if ($request->estado == true) {
					$contratado = 1;
					$ofrecido = 1;
				}
			} else {
				if ($request->estado == true) {
					$ofrecido = 1;
				}
			}

			$caja = new Seguros();
			$caja->id_viaje = $request->id_viaje;
			$caja->id_pasajero = $request->id_pasajero;
			$caja->ofrecido = $ofrecido;
			$caja->contratado = $contratado;
			$caja->costo = 0;
			$caja->pago_prestadora = 0;
			$caja->save();
		} else {
			if ($request->tipo == "contratado") {
				if ($request->estado == false) {
					DB::table('seguros')->where('id',$request->id_seguro)->update(['contratado'=>0]);
				} else {
					DB::table('seguros')->where('id',$request->id_seguro)->update(['ofrecido'=>1, 'contratado'=>1]);
				}
			} else {
				if ($request->estado == true) {
					DB::table('seguros')->where('id',$request->id_seguro)->update(['ofrecido'=>1]);
				} else {
					DB::table('seguros')->where('id',$request->id_seguro)->update(['ofrecido'=>0]);
				}
			}
		}
	}
	
    public function getExcel($id) {

		$dato_viaje = Viaje::find($id);

		$viaje = $dato_viaje->destino->ciudad;
        $viaje = str_replace(",", "", $viaje);
        $viaje_url = str_replace(" ", "-", $viaje);
		$fecha_viaje = date('d-m-Y', strtotime($dato_viaje->fecha_hora_salida));

		$sumatoria_tarifa = 0;
		$sumatoria_entregado = 0;
		$sumatoria_saldo = 0;

		$seguros = DB::table('seguros')->where('id_viaje',$id)->get();

		$reservas = ReservaInfo::where('reserva_info.viaje_id', $id)
								->join('pasajero', 'pasajero.id', '=', 'reserva_info.pasajero_id')
								->leftJoin('seguros', 'seguros.id_pasajero', '=', 'reserva_info.pasajero_id')
								->where('pasajero.coordinador', '!=', 1)
								->select(
									'pasajero.id as pasajero_id',
									'pasajero.nombre',
									'pasajero.apellido',
									'reserva_info.id as id_reserva_info',
									'seguros.ofrecido as ofrecido_seguro',
									'seguros.contratado as contratado_seguro',
									'seguros.costo as costo_seguro',
									'seguros.pago_prestadora',
									'seguros.id as id_seguro',
								)
								->get()->all();

		$filename ="Seguros-".$viaje_url."-(".$fecha_viaje.").xls";
		header('Content-type: application/ms-excel');
		header('Content-Disposition: attachment; filename='.$filename);

		$stuff="SEGUROS AL VIAJERO\n".$viaje."\t(".$fecha_viaje.")\t\t\n";
		$stuff=$stuff."\n";
		$stuff=$stuff."\tPASAJERO\tOFRECIDO\tCONTRATADO\tTARIFA\tENTREGADO\tSALDO\n";

		$stuff=$stuff."\n";

		foreach ($reservas as $reserva) {
			$tarifa = $reserva->costo_seguro;

			$entregado = 0;
			$cuotas = DB::table('cuotas_seguros')
						->where([['id_viaje',$id],['id_pasajero',$reserva->pasajero_id]])
						->join('caja2020', 'caja2020.id', '=', 'cuotas_seguros.id_caja')
						->select('caja2020.monto as cuota', 'caja2020.tipo_transaccion_id', 'caja2020.monto_cambio')
						->get();
			
			foreach ($cuotas as $cuota) {
				$monto_cambio = '1';
				if (($cuota->tipo_transaccion_id == "5" || $cuota->tipo_transaccion_id == "8" || $cuota->tipo_transaccion_id == "13") && is_numeric($cuota->monto_cambio)) {
					$monto_cambio = $cuota->monto_cambio;
				}
				$entregado += $cuota->cuota * $monto_cambio;
			}

			$saldo = $tarifa - $entregado;
			$sumatoria_tarifa += $tarifa;
			$sumatoria_entregado += $entregado;
			$sumatoria_saldo += $saldo;

			$stuff=$stuff."\t".$reserva->apellido." ".$reserva->nombre."\t";
			if ($reserva->ofrecido_seguro == 1) {
				$stuff=$stuff."SI\t";
			} else {
				$stuff=$stuff."NO\t";
			}
			if ($reserva->contratado_seguro == 1) {
				$stuff=$stuff."SI\t";
			} else {
				$stuff=$stuff."NO\t";
			}
			$stuff=$stuff."\t$".number_format($tarifa, 2, '.', '')."\t$".number_format($entregado, 2, '.', '')."\t$".number_format($saldo, 2, '.', '')."\n";

		}

		$stuff=$stuff."\n\tTOTAL TARIFA\t$".number_format($sumatoria_tarifa, 2, '.', '')."\n\tTOTAL ENTREGADO\t$".number_format($sumatoria_entregado, 2, '.', '')."\n\tTOTAL SALDO\t$".number_format($sumatoria_saldo, 2, '.', '')."\n";

        echo $stuff;        
    }
}