<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\model\sistema\Viaje;
use App\model\sistema\DataWeb;
use Illuminate\Support\Facades\Mail;
use App\Mail\SendContactMessage;
use App\model\sistema\ImagenesFlyers;
use App\model\sistema\ImagenesSlider;
use App\model\sistema\ListaVendedores;

class Site extends Controller
{
    public function index() {

        $anios = date('Y') - 2018;
        $cant_viajes = Viaje::where([
            ['deleted', '=', 0],
            ['estado', '!=', 1]
        ])->whereDate('fecha_hora_retorno', '<', date('Y-m-d H:i:s'))->count();

        $data_web = DataWeb::find('1');
        $imagen_portada = ImagenesSlider::orderByRaw('RAND()')->take(1)->value('path');
        $imagen_bg_nosotros = ImagenesSlider::orderByRaw('RAND()')->take(1)->value('path');
        $imagen_bg_contacto = ImagenesSlider::orderByRaw('RAND()')->take(1)->value('path');

        $viajes = $this->getViajes();
        $categorias = $viajes->pluck('categoria.nombre')->unique()->sort()->values()->toArray();

        //return view('site/indexUnderConstruction', ['viajes'=>$viajes,'anios'=>$anios,'categorias'=>$categorias,'data_web'=>$data_web,'cant_viajes'=>$cant_viajes,'imagen_portada'=>$imagen_portada]);
        return view('site/index', [
            'viajes'=>$viajes,
            'anios'=>$anios,
            'categorias'=>$categorias,
            'data_web'=>$data_web,
            'cant_viajes'=>$cant_viajes,
            'imagen_portada'=>$imagen_portada,
            'imagen_bg_nosotros'=>$imagen_bg_nosotros,
            'imagen_bg_contacto'=>$imagen_bg_contacto
        ]);
    }

    public function circuito($id) {
        $data_web = DataWeb::find('1');
        $viaje = Viaje::where('id', $id)
            ->with('destino.image', 'incluye', 'categoria')
            ->first();

        switch ($viaje->moneda) {
            case '5':
                $moneda = 'us$';
                break;
            case '8':
                $moneda = '€';
                break;
            case '13':
                $moneda = 'R$';
                break;
            default:
                $moneda = '$';
                break;
        }

        $viajes_random = $this->getFourViajes($id);

        return view('site/viajeInfo', ['data_web'=>$data_web,'viaje'=>$viaje,'moneda'=>$moneda,'viajes_random'=>$viajes_random]);
    }

    public function circuitos(Request $request) {
        $busqueda = false;
        $dataBusqueda = array();
        if (isset($request->tipo_viaje)) {
            $busqueda = true;
            $dataBusqueda = array('tipo_viaje'=>$request->tipo_viaje, 'fecha_desde'=>$request->fecha_desde, 'fecha_hasta'=>$request->fecha_hasta);

            $fecha_desde_format = explode('/', $request->fecha_desde);
            $fecha_desde_format = $fecha_desde_format[2].'-'.$fecha_desde_format[1].'-'.$fecha_desde_format[0];
            $fecha_hasta_format = explode('/', $request->fecha_hasta);
            $fecha_hasta_format = $fecha_hasta_format[2].'-'.$fecha_hasta_format[1].'-'.$fecha_hasta_format[0];

            $viajes = $this->getViajesSearch($request->tipo_viaje, $fecha_desde_format, $fecha_hasta_format);
        } else {
            $viajes = $this->getViajes();
        }

        $data_web = DataWeb::find('1');
        $imagenes_flyers = ImagenesFlyers::orderBy('id', 'asc')->get();
        //$categorias = $viajes->pluck('categoria.nombre')->unique()->sort()->values()->toArray();

        $array_meses_es = ['','Enero','Febrero','Marzo','Abril','Mayo','Junio','Julio','Agosto','Septiembre','Octubre','Noviembre','Diciembre'];
        $meses_viajes = $viajes->pluck('fecha_hora_salida')->unique()->sort()->map(function ($fecha) use ($array_meses_es) {
            $numero_mes = date('m', strtotime($fecha));
            $nombre_mes = $array_meses_es[(int)$numero_mes];
        
            return $nombre_mes . ' ' . date('Y', strtotime($fecha));
        })->values()->toArray();
        $meses_viajes = array_unique($meses_viajes);

        $viajesPorCategoria = $viajes->groupBy(function ($viaje) {
            return $viaje->categoria->nombre;
        })->map(function ($viajes) {
            return $viajes->count();
        });

        return view('site/circuitos', [
            'data_web'=>$data_web,
            'viajes'=>$viajes,
            'meses_viajes'=>$meses_viajes,
            'busqueda'=>$busqueda,
            'dataBusqueda'=>$dataBusqueda,
            'imagenes_flyers'=>$imagenes_flyers,
            'viajesPorCategoria'=>$viajesPorCategoria
        ]);
    }

    public function vendedores() {
        $data_web = DataWeb::find('1');
        $vendedores = ListaVendedores::orderBy('nombre', 'asc')->get();
        $imagen_bg_contacto = ImagenesSlider::orderByRaw('RAND()')->take(1)->value('path');
        return view('site/vendedores', ['data_web'=>$data_web,'vendedores'=>$vendedores,'imagen_bg_contacto'=>$imagen_bg_contacto]);
    }

    public function contacto() {
        $data_web = DataWeb::find('1');
        return view('site/contacto', ['data_web'=>$data_web]);
    }

    /* public function getViajes() {
        $query = Viaje::where([
                ['mostrar_web', '=', 1],
                ['deleted', '=', 0],
                ['estado', '=', 2],
            ])
            ->whereDate('fecha_hora_salida', '>', date('Y-m-d'))
            ->with('categoria')
            ->with('destino.image');

        return $query->orderBy('fecha_hora_salida', 'asc')->get();
    } */
    public function getViajes() {
        $query = Viaje::where([
                ['mostrar_web', '=', 1],
                ['deleted', '=', 0],
            ])
            ->whereIn('estado', [2, 4])
            ->whereDate('fecha_hora_salida', '>', date('Y-m-d'))
            ->with('categoria')
            ->with('destino.image');
    
        return $query->orderBy('fecha_hora_salida', 'asc')->get();
    }
    public function getFourViajes($id = '') {
        if (!empty($id)) {
            $query = Viaje::where([
                    ['mostrar_web', '=', 1],
                    ['deleted', '=', 0],
                    ['estado', '=', 2],
                    ['id', '!=', $id],
                ])
                ->whereDate('fecha_hora_salida', '>', date('Y-m-d'))
                ->with('categoria')
                ->with('destino.image');
        } else {
            $query = Viaje::where([
                    ['mostrar_web', '=', 1],
                    ['deleted', '=', 0],
                    ['estado', '=', 2],
                ])
                ->whereDate('fecha_hora_salida', '>', date('Y-m-d'))
                ->with('categoria')
                ->with('destino.image');
        }

        return $query->inRandomOrder()->limit(4)->get();
    }

    public function getViajesSearch($tipo, $desde, $hasta) {
        $query = Viaje::where([
                ['mostrar_web', '=', 1],
                ['deleted', '=', 0],
                ['estado', '=', 2],
            ])
            ->whereDate('fecha_hora_salida', '>=', $desde)
            ->whereDate('fecha_hora_salida', '<=', $hasta)
            ->with('categoria')
            ->with('destino.image');

        if ($tipo !== 'all') {
            $query->whereHas('categoria', function ($categoriaQuery) use ($tipo) {
                $categoriaQuery->where('nombre', $tipo);
            });
        }

        return $query->orderBy('fecha_hora_salida', 'asc')->get();
    }

    public function sendContactMessage(Request $request) {
        $configuraciones = DataWeb::find('1');
        if (!empty($configuraciones->email)) {
            if (!empty($request->email)) {
                $data = [
                    'nombre'=>$request->name,
                    'email'=>$request->email,
                    'phone'=>$request->phone,
                    'mensaje'=>$request->message
                ];
                Mail::to($configuraciones->email)->send(new SendContactMessage($data));
                return 1;
            }
        }
        return 0;
    }
}
