<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\model\sistema\Users;
use App\model\sistema\Reserva;
use App\model\sistema\ReservaInfo;
use Validator;
use Illuminate\Validation\Rule;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\DB;

class UsersController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth');
        $this->usersModel = new Users;
    }

    public function index(Request $request)
    {
        $ruta = 'usuarios';
        $titulo = 'USUARIOS';
        if ($request->query('tipo') == 'vendedores') {
            $ruta = 'vendedores';
            $titulo = 'VENDEDORES';
        }
        $users = $this->getAll();

        return view('users', ['title' => $titulo, 'users' => $users, 'ruta' => $ruta]);
    }

    public static function getAll()
    {
        $users = Users::where([
            ['estado', '=', 1],
            ['deleted_at', '=', null]
        ])->get()->all();
        foreach ($users as $user) {
            unset($user->password);
        }
        return $users;
    }

    public function validateAndSave(Request $request)
    {
        $user = $request['user'];
        $rules = [
            'name' => 'required|string|max:255',
            'lastName' => 'required|string|max:255',
            'nroTelefono' => 'required',
        ];
        if (array_key_exists('id', $user)) {
            $rules['email'] = [
                'required',
                'email',
                'max:255',
                Rule::unique('users')->ignore($user['id'])
            ];
            if ($user['password'] !== null) {
                $rules['password'] = 'required|string|min:6';
                $rules['passwordRepeat'] = 'required|string|min:6|same:password';
            }
        } else {
            $rules['email'] = 'required|email|max:255|unique:users';
            $rules['password'] = 'required|string|min:6';
            $rules['passwordRepeat'] = 'required|string|min:6|same:password';
        }

        $validator = Validator::make($user, $rules, []);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()->all()]);
        } else {
            unset($user['passwordRepeat']);
            $user = $this->saveUser($user);
            return json_encode($user);
        }
    }

    private function saveUser($data)
    {
        $user = array_key_exists('id', $data) ? Users::find($data['id']) : new Users;
        $user->name = $data['name'];
        $user->lastName = $data['lastName'];
        $user->dni = $data['dni'];
        $user->role = (int)$data['role'];
        $user->email = $data['email'];
        $user->nroTelefono = $data['nroTelefono'];
        $user->password = Hash::make($data['password']);
        $user->save();
        return $user;
    }

    public function disable(Request $request)
    {
        $id = $request['id'];
        return Users::where('id', $request['id'])
            ->update(['estado' => 0]);
    }

    public function disabled()
    {
        $disabledUsers = Users::where([
            ['estado', '=', 0],
            ['deleted_at', '=', null]
        ])->get()->all();
        return view('disabledusers', ['title' => 'USUARIOS DESHABILITADOS', 'users' => $disabledUsers]);
    }

    public function enable(Request $request)
    {
        $id = $request['id'];
        return Users::where('id', $request['id'])
            ->update(['estado' => 1]);
    }

    public function getUserInfo($id)
    {
        $data = [];
        $data['user'] = Users::find($id);
        unset($data['user']->password);
        $data['reservas'] = Reserva::where('user_id', $id)
            ->with('info', 'info.pasajero', 'viaje')
            ->get()->all();

        $data_reserva_comisiones_cobradas = DB::table('reserva_info')
            ->join('comisiones', 'comisiones.id_reserva_info', '=', 'reserva_info.id')
            ->select('comisiones.*', 'reserva_info.pasajero_id')
            ->where('comisiones.id_user', $id)
            ->get();

        $data_comisiones_cobradas = $data_reserva_comisiones_cobradas->map(function ($item) {
            $caja = DB::table('caja2020')->where('id', $item->id_caja)->first();
            $data_user = DB::table('users')->where('id', $item->id_user)->first();
            $vendedor = $data_user->lastName . ", " . $data_user->name;
            $data_user_pago = DB::table('users')->where('id', $item->pago_comision)->first();
            $user_pago = $data_user_pago->lastName . ", " . $data_user_pago->name;
            $data_pasajero = DB::table('pasajero')->where('id', $item->pasajero_id)->first();
            $pasajero = $data_pasajero->apellido . ", " . $data_pasajero->nombre;
            $data_reserva_info = DB::table('reserva_info')->where('id', $item->id_reserva_info)->first();
            $id_viaje = $data_reserva_info->viaje_id;
            $data_viaje = DB::table('viaje')->where('id', $id_viaje)->first();
            $data_destino = DB::table('destino')->where('id', $data_viaje->destino_id)->first();
            $viaje = $data_destino->ciudad . ' (' . date('d/m/Y', strtotime($data_viaje->fecha_hora_salida)) . ')';
            return [
                'id' => $item->id,
                'id_user' => $item->id_user,
                'id_caja' => $item->id_caja,
                'vendedor' => $vendedor,
                'pago_comision' => $user_pago,
                'pasajero' => $pasajero,
                'montoSinFormat' => $item->monto,
                'monto' => number_format($item->monto, 2, ',', '.'),
                'moneda' => ($item->tipo_transaccion_id == "6") ? '$' : 'U$S',
                'tipo_transaccion_id' => $item->tipo_transaccion_id,
                'monto_cambio' => $caja->monto_cambio,
                'viaje' => $viaje,
                'fecha' => $item->datetime,
                'fechaFormat' => date('d/m/Y', strtotime($item->datetime)),
            ];
        });

        $data_reserva_comisiones_por_cobrar = DB::table('reserva_info')
            ->join('reserva', 'reserva.id', '=', 'reserva_info.reserva_id')
            ->join('viaje', 'viaje.id', '=', 'reserva_info.viaje_id')
            ->select('reserva_info.*', 'reserva.user_id', 'viaje.fecha_hora_retorno', 'viaje.destino_id', 'viaje.fecha_hora_salida')
            ->where([['reserva.user_id', $id], ['reserva_info.cobro_comision', 0], ['fecha_hora_retorno', '>=', date('2023-09-01 00:00:00')]])
            ->get();

        $data_comisiones_por_cobrar = $data_reserva_comisiones_por_cobrar->map(function ($item) {
            $data_pasajero = DB::table('pasajero')->where('id', $item->pasajero_id)->first();
            $pasajero = $data_pasajero->apellido . ", " . $data_pasajero->nombre;
            $data_destino = DB::table('destino')->where('id', $item->destino_id)->first();
            $viaje = $data_destino->ciudad . ' (' . date('d/m/Y', strtotime($item->fecha_hora_salida)) . ')';
            return [
                'id' => $item->id,
                'pasajero' => $pasajero,
                'viaje' => $viaje,
            ];
        });

        return view('user-info', [
            'title' => 'Usuario',
            'data' => $data,
            'data_comisiones_cobradas' => $data_comisiones_cobradas,
            'data_comisiones_por_cobrar' => $data_comisiones_por_cobrar
        ]);
    }

    public function editar_usuario(Request $request)
    {
        $usuarioID = $request->usuarioID;
        $nombre = $request->nombre;
        $apellido = $request->apellido;
        $dni = $request->dni;
        $telefono = $request->telefono;
        $role = $request->role;
        $email = $request->email;
        $new_password = $request->new_password;

        if (empty($new_password)) {
            Users::find($usuarioID)->update(['name' => $nombre, 'lastName' => $apellido, 'dni' => $dni, 'role' => $role, 'email' => $email, 'nroTelefono' => $telefono, 'updated_at' => date('Y-m-d H:i:s')]);
        } else {
            $password = Hash::make($new_password);
            Users::find($usuarioID)->update(['name' => $nombre, 'lastName' => $apellido, 'dni' => $dni, 'role' => $role, 'email' => $email, 'nroTelefono' => $telefono, 'updated_at' => date('Y-m-d H:i:s'), 'password' => $password]);
        }

        return back();
    }

    public function exportarExcel($id, $tipo)
    {

        $dataUser = DB::table('users')->where('id', $id)->first();
        $nombreUsuario = $dataUser->lastName . ' ' . $dataUser->name;
        $nombreUsuarioURL = str_replace(' ', '-', $nombreUsuario);

        $stuff = "";

        /* Reservas */
        if ($tipo == 1) {
            $reservas = Reserva::where('user_id', $id)
                ->with('info.pasajero', 'viaje.destino')
                ->orderBy('id', 'ASC')->get()->all();

            $filename = "Reservas-" . $nombreUsuarioURL . ".xls";
            header('Content-type: application/ms-excel');
            header('Content-Disposition: attachment; filename=' . $filename);

            $stuff = "RESERVAS\n" . $nombreUsuario . "\n\nCantidad de Reservas:\t" . count($reservas) . "\n";
            $stuff = $stuff . "\n";
            $stuff = $stuff . "\t\tCÓDIGO\tFECHA RESERVA\tPASAJEROS\tDESTINO\tFECHA SALIDA\n";

            $stuff = $stuff . "\n";

            foreach ($reservas as $dato) {

                $nombre_pasajero = "";
                foreach ($dato->info as $pasajero) {
                    $nombre_pasajero = (empty($nombre_pasajero)) ? $pasajero->pasajero->apellido . ", " . $pasajero->pasajero->nombre : $nombre_pasajero . ' - ' . $pasajero->pasajero->apellido . ", " . $pasajero->pasajero->nombre;
                }

                $stuff = $stuff . "\t\t" . $dato->id . "\t" . date('d/m/Y', strtotime($dato->created_at)) . "\t" . $nombre_pasajero . "\t" . $dato->viaje->destino->ciudad . "\t" . date('d/m/Y', strtotime($dato->viaje->fecha_hora_salida)) . "\n";
            }
        }

        /* Comisiones Cobradas */
        if ($tipo == 2) {

            $data_reserva = DB::table('reserva_info')
                ->join('comisiones', 'comisiones.id_reserva_info', '=', 'reserva_info.id')
                ->select('comisiones.*', 'reserva_info.pasajero_id')
                ->where('comisiones.id_user', $id)
                ->orderBy('datetime', 'asc')
                ->get();

            $data = $data_reserva->map(function ($item) {
                $data_user_pago = DB::table('users')->where('id', $item->pago_comision)->first();
                $user_pago = $data_user_pago->lastName . ", " . $data_user_pago->name;
                $data_pasajero = DB::table('pasajero')->where('id', $item->pasajero_id)->first();
                $pasajero = $data_pasajero->apellido . ", " . $data_pasajero->nombre;
                $data_reserva_info = DB::table('reserva_info')->where('id', $item->id_reserva_info)->first();
                $id_viaje = $data_reserva_info->viaje_id;
                $data_viaje = DB::table('viaje')->where('id', $id_viaje)->first();
                $data_destino = DB::table('destino')->where('id', $data_viaje->destino_id)->first();
                $viaje = $data_destino->ciudad . ' (' . date('d/m/Y', strtotime($data_viaje->fecha_hora_salida)) . ')';
                return [
                    'fecha' => date('d/m/Y', strtotime($item->datetime)),
                    'pago_comision' => $user_pago,
                    'pasajero' => $pasajero,
                    'viaje' => $viaje,
                    'monto' => number_format($item->monto, 2, ',', '.'),
                    'montoSinFormat' => $item->monto,
                ];
            });

            $ganancia_total = 0;
            foreach ($data as $dato) {
                $ganancia_total = $ganancia_total + $dato['montoSinFormat'];
            }
            $ganancia_total = number_format($ganancia_total, 2, ',', '.');

            $filename = "Comisiones-Cobradas-" . $nombreUsuarioURL . ".xls";
            header('Content-type: application/ms-excel');
            header('Content-Disposition: attachment; filename=' . $filename);

            $stuff = "COMISIONES COBRADAS\n" . $nombreUsuario . "\n\nCantidad de Comisiones:\t" . count($data) . "\nGanancia Total:\t$" . $ganancia_total . "\n";
            $stuff = $stuff . "\n";
            $stuff = $stuff . "\t\tFECHA DE PAGO\tPAGÓ COMISIÓN\tPASAJERO\tVIAJE\tMONTO ABONADO\n";

            $stuff = $stuff . "\n";

            foreach ($data as $dato) {
                $stuff = $stuff . "\t\t" . $dato['fecha'] . "\t" . $dato['pago_comision'] . "\t" . $dato['pasajero'] . "\t" . $dato['viaje'] . "\t$" . $dato['monto'] . "\n";
            }
        }

        /* Comisiones por Cobrar */
        if ($tipo == 3) {

            $data_reserva = DB::table('reserva_info')
                ->join('reserva', 'reserva.id', '=', 'reserva_info.reserva_id')
                ->join('viaje', 'viaje.id', '=', 'reserva_info.viaje_id')
                ->select('reserva_info.*', 'reserva.user_id', 'viaje.fecha_hora_retorno', 'viaje.destino_id', 'viaje.fecha_hora_salida')
                ->where([['reserva.user_id', $id], ['reserva_info.cobro_comision', 0], ['fecha_hora_retorno', '>=', date('Y-m-d 00:00:00')]])
                ->orderBy('fecha_hora_salida', 'asc')
                ->get();

            $data = $data_reserva->map(function ($item) {
                $data_pasajero = DB::table('pasajero')->where('id', $item->pasajero_id)->first();
                $pasajero = $data_pasajero->apellido . ", " . $data_pasajero->nombre;
                $data_destino = DB::table('destino')->where('id', $item->destino_id)->first();
                $viaje = $data_destino->ciudad . ' (' . date('d/m/Y', strtotime($item->fecha_hora_salida)) . ')';
                return [
                    'pasajero' => $pasajero,
                    'viaje' => $viaje,
                ];
            });

            $filename = "Comisiones-por-Cobrar-" . $nombreUsuarioURL . ".xls";
            header('Content-type: application/ms-excel');
            header('Content-Disposition: attachment; filename=' . $filename);

            $stuff = "COMISIONES POR COBRAR\n" . $nombreUsuario . "\n\nCantidad de Comisiones:\t" . count($data) . "\n";
            $stuff = $stuff . "\n";
            $stuff = $stuff . "\t\tVIAJE\tPASAJERO\n";

            $stuff = $stuff . "\n";

            foreach ($data as $dato) {
                $stuff = $stuff . "\t\t" . $dato['viaje'] . "\t" . $dato['pasajero'] . "\n";
            }
        }

        echo $stuff;
    }

    public function exportarExcelComisiones(Request $request)
    {
        $id = $request->id_user;
        $tipo = $request->tipo;

        $dataUser = DB::table('users')->where('id', $id)->first();
        $nombreUsuario = $dataUser->lastName . ' ' . $dataUser->name;
        $nombreUsuarioURL = str_replace(' ', '-', $nombreUsuario);

        $stuff = "";

        /* Comisiones Cobradas */
        if ($tipo == 2) {
            $inputCobradas = $request->input_cobradas;

            $data_reserva = DB::table('reserva_info')
                ->join('comisiones', 'comisiones.id_reserva_info', '=', 'reserva_info.id')
                ->select('comisiones.*', 'reserva_info.pasajero_id')
                ->where('comisiones.id_user', $id)
                ->orderBy('datetime', 'asc')
                ->get();

            $filtered_data = $data_reserva->filter(function ($item) use ($inputCobradas) {
                return in_array($item->id, $inputCobradas);
            });

            $data = $filtered_data->map(function ($item) {
                $data_user_pago = DB::table('users')->where('id', $item->pago_comision)->first();
                $user_pago = $data_user_pago->lastName . " " . $data_user_pago->name;
                $data_pasajero = DB::table('pasajero')->where('id', $item->pasajero_id)->first();
                $pasajero = $data_pasajero->apellido . " " . $data_pasajero->nombre;
                $data_reserva_info = DB::table('reserva_info')->where('id', $item->id_reserva_info)->first();
                $id_viaje = $data_reserva_info->viaje_id;
                $data_viaje = DB::table('viaje')->where('id', $id_viaje)->first();
                $data_destino = DB::table('destino')->where('id', $data_viaje->destino_id)->first();
                $viaje = $data_destino->ciudad . ' (' . date('d/m/Y', strtotime($data_viaje->fecha_hora_salida)) . ')';

                return [
                    'fecha' => date('d/m/Y', strtotime($item->datetime)),
                    'pago_comision' => $user_pago,
                    'pasajero' => $pasajero,
                    'viaje' => $viaje,
                    'monto' => number_format($item->monto, 2, '.', ' '),
                    'montoSinFormat' => $item->monto,
                ];
            });

            $ganancia_total = 0;
            foreach ($data as $dato) {
                $ganancia_total = $ganancia_total + $dato['montoSinFormat'];
            }
            $ganancia_total = number_format($ganancia_total, 2, '.', ' ');

            $filename = "Comisiones-Cobradas-" . $nombreUsuarioURL . ".xls";
            header('Content-type: application/ms-excel');
            header('Content-Disposition: attachment; filename=' . $filename);

            $stuff = "COMISIONES COBRADAS\n" . $nombreUsuario . "\n\nCantidad de Comisiones:\t" . count($data) . "\nGanancia Total:\t$" . $ganancia_total . "\n";
            $stuff = $stuff . "\n";
            $stuff = $stuff . "\t\tFECHA DE PAGO\tPAGÓ COMISIÓN\tPASAJERO\tVIAJE\tMONTO ABONADO\n";

            $stuff = $stuff . "\n";

            foreach ($data as $dato) {
                $stuff = $stuff . "\t\t" . $dato['fecha'] . "\t" . $dato['pago_comision'] . "\t" . $dato['pasajero'] . "\t" . $dato['viaje'] . "\t$" . $dato['monto'] . "\n";
            }
        }

        /* Comisiones por Cobrar */
        if ($tipo == 3) {
            $inputPorCobrar = $request->input_por_cobrar;

            $data_reserva = DB::table('reserva_info')
                ->join('reserva', 'reserva.id', '=', 'reserva_info.reserva_id')
                ->join('viaje', 'viaje.id', '=', 'reserva_info.viaje_id')
                ->select('reserva_info.*','reserva.user_id','viaje.fecha_hora_retorno','viaje.destino_id','viaje.fecha_hora_salida')
                ->where([['reserva.user_id', $id],['reserva_info.cobro_comision', 0],['fecha_hora_retorno', '>=', date('2023-09-01 00:00:00')]])
                ->get();

            $filtered_data = $data_reserva->filter(function ($item) use ($inputPorCobrar) {
                return in_array($item->id, $inputPorCobrar);
            });
        
            $data = $filtered_data->map(function ($item) {
                $data_pasajero = DB::table('pasajero')->where('id',$item->pasajero_id)->first();
                $pasajero = $data_pasajero->apellido." ".$data_pasajero->nombre;
                $data_destino = DB::table('destino')->where('id',$item->destino_id)->first();
                $viaje = $data_destino->ciudad.' ('.date('d/m/Y', strtotime($item->fecha_hora_salida)).')';
                return [
                    'id'=>$item->id,
                    'pasajero'=>$pasajero,
                    'viaje'=>$viaje,
                ];
            });

            $filename = "Comisiones-por-Cobrar-" . $nombreUsuarioURL . ".xls";
            header('Content-type: application/ms-excel');
            header('Content-Disposition: attachment; filename=' . $filename);

            $stuff = "COMISIONES POR COBRAR\n" . $nombreUsuario . "\n\nCantidad de Comisiones:\t" . count($data) . "\n";
            $stuff = $stuff . "\n";
            $stuff = $stuff . "\t\tVIAJE\tPASAJERO\n";

            $stuff = $stuff . "\n";

            foreach ($data as $dato) {
                $stuff = $stuff . "\t\t" . $dato['viaje'] . "\t" . $dato['pasajero'] . "\n";
            }
        }

        echo $stuff;
    }
}
