<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\model\sistema\Viaje;
use App\model\sistema\Destino;
use App\model\sistema\ReservaInfo;
use App\model\sistema\Pasajero;
use App\model\sistema\Incluye;
use App\model\sistema\CategoriaViaje;
use App\model\sistema\Locked;
use App\model\sistema\Reserva;

use Validator;
use Illuminate\Support\Facades\DB;

class ViajeController extends Controller
{

	protected $viajeModel;

	public function __construct(Request $request) {
        $this->middleware('auth');
    }

    public function index() {
        $destinos = Destino::get();
        $categoria_viaje = CategoriaViaje::all();
        $pasajeros = Pasajero::all();
        return view('viajes', ['title'=>'VIAJES','destinos'=>$destinos,'categoria_viaje'=>$categoria_viaje,'pasajeros'=>$pasajeros]);
    }

    public function get(Request $request) {
        return Viaje::with('destino')->find($request->viajeId);
    }

    public function getAll() {
        $viajes = Viaje::where('viaje.deleted', 0)
            ->whereIn('viaje.estado', [1, 2, 3, 4])
            ->orderBy('viaje.fecha_hora_salida', 'asc')
            ->with('incluye', 'destino', 'categoria', 'habitaciones.type', 'reserva')
            ->get();
        return $viajes->toJson();
    }

    public function getActive($tipo) {
        if ($tipo == "all" or $tipo == "") {
            $viajes = Viaje::where([['viaje.deleted', 0],['individual','!=',1]])
                    ->whereDate('fecha_hora_retorno', '>=', date('Y-m-d'))
                    ->orderBy('viaje.fecha_hora_salida', 'asc')
                    ->with('incluye', 'destino', 'categoria', 'habitaciones.type', 'reserva', 'reservainfo')
                    ->get();
        } else {
            $viajes = Viaje::where([['viaje.deleted', 0],['individual','!=',1]])
                    ->where('viaje.categoria_id', $tipo)
                    ->whereDate('fecha_hora_retorno', '>=', date('Y-m-d'))
                    ->orderBy('viaje.fecha_hora_salida', 'asc')
                    ->with('incluye', 'destino', 'categoria', 'habitaciones.type', 'reserva', 'reservainfo')
                    ->get();
        }        
        return $viajes->toJson();
    }

    // ENVIA LOS DATOS AL MODELO PARA LA CREACION DE UN NUEVO VIAJE
    public function save(Request $request) {
        $viaje = $request['viaje'];

        if (isset($request['viaje']['crear_destino']) && $request['viaje']['crear_destino'] == 1) {
            $rules = $this->createValidationRulesCrearDestino();
            $customMessages = $this->createValidationCustomMessagesCrearDestino();
        } else {
            $rules = $this->createValidationRules();
            $customMessages = $this->createValidationCustomMessages();
        }
        
        $validator = Validator::make($viaje, $rules, $customMessages);
        if($validator->fails()) {
            return response()->json(['errors'=>$validator->errors()->all()]);
        } else {
            return $this->saveViaje($viaje);
        }
    }

    private function createValidationRules() {
        return [
            'destino_id' => 'required',
            'fecha_hora_salida' => 'required|date_format:d/m/Y',
            'fecha_hora_retorno' => 'required|date_format:d/m/Y',
            'moneda' => 'required',
            'precio' => 'required',
            'categoria'=> 'required',
            'tipo_de_coche'=> 'required',
            'noches' => 'required|min:0'
        ];
    }
    private function createValidationRulesCrearDestino() {
        return [
            'ciudad' => 'required',
            'provincia' => 'required',
            'fecha_hora_salida' => 'required|date_format:d/m/Y',
            'fecha_hora_retorno' => 'required|date_format:d/m/Y',
            'moneda' => 'required',
            'precio' => 'required',
            'categoria'=> 'required',
            'tipo_de_coche'=> 'required',
            'noches' => 'required|min:0'
        ];
    }

    private function createValidationCustomMessages() {
        return [
            'destino_id.required' => 'Se debe seleccionar un destino.',
            'fecha_hora_salida.required' => 'La fecha de salida es requerida.',
            'fecha_hora_retorno.required' => 'La fecha de retorno es requerida.'
        ];
    }
    private function createValidationCustomMessagesCrearDestino() {
        return [
            'ciudad.required' => 'Se debe ingresar una ciudad.',
            'provincia.required' => 'Se debe ingresar una provincia.',
            'fecha_hora_salida.required' => 'La fecha de salida es requerida.',
            'fecha_hora_retorno.required' => 'La fecha de retorno es requerida.'
        ];
    }

    private function saveViaje($data) {
        if (isset($data['crear_destino']) && $data['crear_destino'] == 1) {
            $destino = new Destino;
            $destino->ciudad = $data['ciudad'];
            $destino->provincia = $data['provincia'];
            $destino->id_user = auth()->id();
            $destino->save();
        }

        $viaje = array_key_exists('id', $data) ? Viaje::find($data['id']) : new Viaje;
        $viaje->destino_id = (isset($data['crear_destino']) && $data['crear_destino'] == 1) ? $destino->id : $data['destino_id'];
        $viaje->moneda = $data['moneda'];
        $viaje->precio = $data['precio'];
        $viaje->fecha_hora_salida = \DateTime::createFromFormat('d/m/Y', $data['fecha_hora_salida'])->format('Y-m-d H:i:s');
        $viaje->fecha_hora_retorno = \DateTime::createFromFormat('d/m/Y', $data['fecha_hora_retorno'])->format('Y-m-d H:i:s');
        $viaje->categoria_id = $data['categoria'];
        $viaje->tipo_bus = $data['tipo_de_coche'];
        $viaje->aereo = (int) $data['aereo'];
        $viaje->noches = $data['noches'];
        $viaje->id_user = auth()->id();
        if (isset($data['viajeIndividual']) && $data['viajeIndividual'] == 1) {
            $viaje->individual = 1;
        }
        $viaje->mostrar_web = $data['mostrar_web'];
        $viaje->itinerario = $data['itinerario'];
        $viaje->save();
        if (array_key_exists('incluye', $data)) {
            $this->saveIncluyeItems($data['incluye'], $viaje->id);
        }
        DB::table('viaje_tipo_habitacion')->where('viaje_id',$viaje->id)->delete();
        if (array_key_exists('roomTypes', $data)) {
            RoomController::saveRoomTypesForTrip($data['roomTypes'], $viaje->id);
        }
        $viaje->load('destino', 'incluye', 'categoria', 'habitaciones.type');
        return $viaje;
    }

    public function save_foto_portada(Request $request) {

        if ($request->hasFile('new_img')) {
            
            $imagen = $request->file('new_img');
            $newName = time() . '.' . $imagen->getClientOriginalExtension();
            $folderPath = 'images/portadasImages';
            $imagen->move(public_path($folderPath), $newName);

            Viaje::find($request->viajeId)->update(['imagen_portada'=>$folderPath . '/' . $newName]);

        } else {
            $prev_img = Viaje::find($request->viajeId)->value('imagen_portada');
            if (!empty($prev_img)) {
                if (file_exists(public_path($prev_img))) {
                    unlink(public_path($prev_img));
                }
            }
            Viaje::find($request->viajeId)->update(['imagen_portada'=>'']);
        }

        return back();

    }

    private function saveIncluyeItems($items, $viajeId) {
        $incluyeList = [];
        if (array_key_exists('saved', $items)) {
            foreach($items['saved'] as $item) {
                $incluye = array_key_exists('id', $item) ? Incluye::find($item['id']) : new Incluye;
                $incluye->viaje_id = $viajeId;
                $incluye->descripcion = $item['descripcion'];
                $incluye->save();
            }
        }
        if (array_key_exists('deleted', $items)) {
            foreach ($items['deleted'] as $itemId) {
                Incluye::destroy($itemId);
            }
        }
    }

    public function delete(Request $request) {
        $viaje = Viaje::find($request['id']);
        $viaje->deleted = 1;
        $viaje->id_user = auth()->id();
        $viaje->save();
        return 1;
    }

    // CAMBIA EL ESTADO DEL VIAJE
    public function updateStatus(Request $request) {
        $viaje = Viaje::find($request['id']);
        $viaje->estado = $request['newStatus'];
        $viaje->id_user = auth()->id();
        $viaje->save();
        return $request['newStatus'];
    }

    public function bus($viajeId) {

        $ocupados = ReservaInfo::where('viaje_id', $viajeId)->with('pasajero')->get()->keyBy('nro_asiento')->all();
        $bloqueados = Locked::where('viaje_id', $viajeId)->get()->keyBy('butaca')->all();

        $dataArray = array("ocupados" => $ocupados, "bloqueados" => $bloqueados);

        return json_encode($dataArray);

        /* return ReservaInfo::where('viaje_id', $viajeId)
            ->with('pasajero')
            ->get()
            ->keyBy('nro_asiento')->all(); */
    }

    public function getFormData(Request $request) {
        $destinos = Destino::get()->toJson();
        $roomTypes = RoomController::getWithoutSharingRooms()->toJson();
        $categorias = CategoriaViaje::get()->toJson();
        return [
            'destinos' => $destinos,
            'roomTypes' => $roomTypes,
            'categorias' => $categorias
        ];
    }

    public function getDataButOcupadas($id) {
        $but_ocupadas = DB::table('reserva_info')->where('viaje_id',$id)->count();
        return $but_ocupadas;
    }

    public function saveButacaCoordinador($viajeId, Request $request) {
        $reserva = new Reserva;
        $reserva->viaje_id = $viajeId;
        $reserva->user_id = auth()->id();
        $reserva->tipo_habitacion_id = null;
        $reserva->created_at = date('Y-m-d H:i:s');
        $reserva->save();
        
        $info = new ReservaInfo;
        $info->reserva_id = $reserva->id;
        $info->viaje_id = $viajeId;
        $info->pasajero_id = $request->pasajero_id;
        $info->nro_asiento = $request->nro_asiento;
        $info->precio_opcional = 0;
        $info->save();
        
        return $reserva;
    }
}
