class caja_Form {
	constructor(isIncome, transaction, dateSelected) {
		this.transaction = transaction;
		this.isIncome = isIncome;
        this.dateSelected = dateSelected;
		this.getData().then(this.onDataReady.bind(this));
		this.render();
	}

	render() {
		$('#sidebar_right_errors').hide();
		$('#rightsidebarContainer').html($('<div>', {class: 'LoaderContainer'}).append(new Loader()));
		this.setTitle();

	    this.form = $('<form>', {id: "formIngreso"}).append([
	    	$('<div>', {class: "form-group"}).append([
	            $('<div>', {class: "form-line"}).append([
	                $('<input>', {type: "text", class: "nroFactura form-control", placeholder: "Nro Factura"}).val(this.transaction ? this.transaction.nro_factura : null)
	            ])
	        ]),
	        $('<div>', {class: "form-group"}).append([
	            $('<div>', {class: "form-line"}).append([
	                $('<input>', {type: "text", class: "concepto form-control", placeholder: "Concepto"}).val(this.transaction ? this.transaction.concepto : null)
	            ])
	        ]),
            $('<div>', {class:"input-group"}).append([
                $('<div>', {class:"form-line"}).append([
                    $('<span>', {class:"input-group-addon"}).html(this.getCurrencySymbol()),
                    $('<input>', {type:"text", class:"monto form-control", placeholder: "Monto"}).val(this.transaction ? formatMoney(this.transaction.monto) : null)
                ])
            ]),
            this.createCambioInputIfNeeded(),
	        $('<div>', {class: "input-group date", id: "fechaTransaccion"}).append([
	            $('<div>', {class: "form-line"}).append([
	                this.createInputDatePicker()
	            ]),
	            $('<span>', {class: "input-group-addon"}).html($('<i>', {class: "material-icons"}).html('date_range'))
	        ]),
	        this.createMedioPagoSelect(),
	        $('<div>', {class: "form-group"}).append([
	            $('<div>', {class: "form-line"}).append([
	                this.oficinaSelect = $('<select>', {name: "oficina", id: "oficina", class: "form-control"})
	                	.html('<option hidden disabled selected>Elija oficina</option>')
	            ])
	        ]),
	        $('<div>', {class: "form-group", style: "text-align: right"}).append([
	            this.saveButton = $('<button>', {class: "btn btn-success"}).html('Guardar'), ' ', 
	            $('<button>', {class: "btn btn-danger", id: "sidebar_right_cancelar"}).html('Cancelar')
	        ])
	    ])

	    this.saveButton.on('click', this.onClickSave.bind(this));
	}

	setTitle() {
	    $('#sidebar_right_title').html(this.isIncome ? 'REGISTRAR INGRESO' : 'REGISTRAR EGRESO');
	}

	createMedioPagoSelect() {
		return $('<div>', {class: "form-group"}).append([
            $('<div>', {class: "form-line"}).append([
                this.medioPagoSelect = $('<select>', {name: "medio_pago", id: "medio_pago", class:"form-control"})
                	.html('<option hidden disabled selected>Elija tipo de transaccion</option>')
            ])
        ]);
	}

	createInputDatePicker() {
        var fecha = moment(this.transaction ? this.transaction.fecha : this.dateSelected).format('DD/MM/YYYY');
		var picker = $('<input>', {type: "text", class: "fecha form-control", placeholder:"Fecha"})
            .datepicker({
                autoclose: true,
                format: 'dd/mm/yyyy',
                container: '#fechaTransaccion',
                language: 'es',
                'setDate': fecha
            })
        picker.datepicker('setDate', fecha);
        return picker;
    }

    getData() {
    	return $.ajax({
    		url: base_url + '/caja/getformdata',
    		type: 'GET'
    	});	
    }

    get() {
        return this.form;
    }

    onDataReady(response) {
    	this.oficinas = JSON.parse(response[0]);
    	this.mediosPago = this.filterTransactionTypes(JSON.parse(response[1]));
    	this.appendOficinasOptions();
    	this.appendMediosPagoOptions();
        $('.monto').mask("#.##0,00", {reverse: true});
        $('.montoCambio').mask("#.##0,00", {reverse: true});
    	$('#rightsidebarContainer').html(this.form);
    }

    appendOficinasOptions() {
    	var options = [];
    	this.oficinas.forEach(function(oficina) {
    		var option = $('<option>', {value: oficina.id}).html(oficina.nombre);
    		options.push(option);
    	}, this);
    	this.oficinaSelect.append(options);
        if (this.transaction) {
            this.oficinaSelect.val(this.transaction.oficina_id);
        }
    }

    appendMediosPagoOptions() {
    	var options = [];
    	this.mediosPago.forEach(function(medioPago) {
    		var option = $('<option>', {value: medioPago.id}).html(medioPago.nombre);
    		options.push(option);
    	}, this);
    	this.medioPagoSelect.append(options);
        if (this.transaction) {
            this.medioPagoSelect.val(this.transaction.tipo_transaccion_id);
        }
    }

    filterTransactionTypes(transactions) {
    	return transactions.filter(function(transaction) {
    		if (this.isIncome) {
    			return transaction.id != 6 && transaction.id != 7;
    		} else {
    			return transaction.id == 6 || transaction.id == 7;
    		}
    	}, this);
    }

    onClickSave() {
		event.preventDefault();
        if (!this.transaction) {
            this.transaction = {};
        }
		this.transaction.nro_factura = $('.nroFactura').val();
    	this.transaction.concepto = $('.concepto').val();
    	this.transaction.monto = stringToFloatDecimals($('.monto').val());
		this.transaction.fecha = moment($('.fecha').val(), 'DD/MM/YYYY').format('YYYY-MM-DD');
		this.transaction.medio_pago = $('#medio_pago').val();
		this.transaction.oficina_id = $('#oficina').val();

        if (this.transaction.monto_cambio) {
            this.transaction.monto_cambio = stringToFloatDecimals($('.montoCambio').val());
        }
		this.saveTransaction();
    }

    saveTransaction() {
    	$.ajax({
    		url: base_url + '/caja/save',
    		type: 'POST',
    		data: {
    			_token: token,
    			transaction: this.transaction
    		},
    		success: this.onAttemptSaveTransaction.bind(this)
    	});
    }

    onAttemptSaveTransaction(response) {
    	if (response.errors) {
            showErrors('#sidebar_right_errors', response.errors);
        } else {
        	swal('Hecho', 'Transaccion guardado con \u00E9xito.','success');
            this.transaction = response;
            this.form.trigger('updatetransaction', [this.transaction]);
        }
    }

    createCambioInputIfNeeded() {
        if (!this.transaction || !this.transaction.monto_cambio) {
            return;
        }
        return $('<div>', {class:"input-group"}).append([
            $('<div>', {class:"form-line"}).append([
                $('<span>', {class:"input-group-addon"}).html('$'),
                $('<input>', {type:"text", class:"montoCambio form-control", placeholder: "Tipo de Cambio"}).val(formatMoney(this.transaction.monto_cambio))
            ])
        ])
    }

    getCurrencySymbol() {
        if (!this.transaction) {
            return '$';
        }
        var currency = this.transaction.viaje ? this.transaction.viaje.moneda : this.transaction.tipo_transaccion_id;
        return sistema_TipoTransaccion.getCurrencySymbol(currency);
    }
}