class caja_LibroDiario {
	constructor() {
		this.date = null;
		this.goToDate();
		this.render();
	}

	render() {
		this.tableBody = $('<tbody>', {'class': 'LibroDiarioBody'}).append(this.createLoader())
		this.table = $('<table>', {'class': 'table table-striped tabla-circuitos LibroDiario'}).append(
			[$('<thead>').append([
				$('<tr>').append([
					$('<th>').html('Nro Factura'),
					$('<th>').html('Concepto'),
					$('<th>').html('Viaje'),
					$('<th>').html('Forma de Pago'),
					$('<th>').html('Haber'),
					$('<th>').html('Debe'),
					$('<th>').html('Oficina'),
					$('<th>', {class: 'LibroDiarioActionsColumn'}).html('')
				])
			]),
			this.tableBody
		]);

		this.table.on('updatetransaction', this.onUpdateTransaction.bind(this));
		this.table.on('removetransaction', this.onClickRemove.bind(this));


		return this.table;
	}

	goToDate(date) {
		if (this.tableBody) {
			$.fn.DataTable.isDataTable( '.LibroDiario' ) && this.table.DataTable().destroy();

			this.tableBody.html(this.createLoader());
		}
		this.resumeTable && this.resumeTable.get().remove();
		this.date = date || moment().format('YYYY-MM-DD');
		this.transactions = this.getDateTransactions().then(this.onTransactiosReady.bind(this));
	}

	getDateTransactions() {
		return $.ajax({
			url: base_url+'/caja/getdatetransactions',
			type: 'GET',
			data: {
				date: this.date
			}
		});
	}

	getTable() {
		return this.table;
	}

	getResumeTable() {
		return this.resumeTable;
	}

	onTransactiosReady(response) {
		this.transactions = JSON.parse(response);
		this._appendTransactions();
		this.createResumeTable();
	}

	initializeDataTable() {
		this.table.DataTable({
	        bPaginate: false,
	    	bLengthChange: false,
	    	bInfo: false,
	        language: {
	            url: base_url+'/../js/vendors/dataTables/spanish.json'
	        }
		});
	}

	_appendTransactions() {
		if (this.transactions.length > 0) {
			this.tableBody.html('');
			this.transactions.forEach(function(transaction) {
				this.tableBody.append(this._createRow(transaction));
			}, this);
			this.initializeDataTable();
		} else {
			this._appendEmptyMessage();
		}
	}

	_createRow(transaction) {
		return new caja_LibroDiarioRow(transaction);
	}

	appendUniqueColumn(content) {
		return $('<tr>').append(
			$('<td>', {
				'class': 'uniqueColumn',
				'colspan': 5
			}).append(content)
		);
	}

	_appendEmptyMessage() {
		var message = this.appendUniqueColumn($('<span>').html('No hay transacciones ingresadas en esta fecha.'))
		this.tableBody.html(message);
	}

	createLoader() {
		return this.appendUniqueColumn(new Loader);
	}

	createResumeTable() {
		this.resumeTable = new caja_ResumeTable(this.transactions);
		$('.RightPanel').append(this.resumeTable.get());
	}

	onClickRemove(event, transactionId) {
		event.preventDefault();
        swal({
            title: 'Borrar Transaccion',
            text: 'Esta seguro que desea borrar esta transaccion?',
            icon: 'warning',
            buttons: ['Cancelar', 'Aceptar'],
        }).then((value) => {
            if(value) {
                $.ajax({
                    url: base_url+'/caja/remove',
                    type: 'POST',
                    data: {
                        _token: $('meta[name="csrf-token"]').attr('content'),
                        id: transactionId
                    },
                    success: this.onEndRemove.bind(this, transactionId)
                })
            }
        })
	}

	onEndRemove(transactionId) {
		swal('Hecho', 'Transaccion borrada con \u00E9xito.', 'success');
		this.transactions.forEach(function(transaction, index) {
			if (transaction.id == transactionId) {
				this.transactions.splice(index, 1);
			}
		}, this);

		this.removeRowFromDataTable(transactionId);

		this.transactions.length == 0 && this._appendEmptyMessage();
		this.resumeTable.updateMontos();
	}

	appendNewTransactionIfNeeded(transaction) {
		if (!moment(this.date).isSame(transaction.fecha)) {
			return;
		}
		if (this.transactions.length == 0) {
			this.tableBody.html('');
		}
		this.transactions.push(transaction);

		var newRow = this._createRow(transaction);
		if ($.fn.DataTable.isDataTable( '.LibroDiario' )) {
			this.table.DataTable().row.add(newRow[0]).draw();
		} else {
			this.tableBody.append(newRow);
			this.initializeDataTable(); 
		}

		this.resumeTable.updateMontos();
	}

	onUpdateTransaction(event, transactionUpdated) {
		this.transactions.forEach(function(transaction, index) {
			if (transaction.id == transactionUpdated.id) {
				if (!moment(this.date).isSame(transactionUpdated.fecha)) {

					this.transactions.splice(index, 1);
					this.removeRowFromDataTable(transactionUpdated.id);
					this.transactions.length == 0 && this._appendEmptyMessage();

				} else {
					this.transactions.splice(index, 1, transactionUpdated);
					this.table.DataTable()
						.row($('.rowTransaction' + transactionUpdated.id)).invalidate().draw();
				}
			}
		}, this);
		this.resumeTable.updateMontos();
	}

	removeRowFromDataTable(transactionId) {
		this.table.DataTable()
			.row($('.rowTransaction' + transactionId))
			.remove()
			.draw();
	}

	toggleHeight() {
		this.tableBody.toggleClass('CollapseHeight');
	}
}