class cuentas_FormCuotas {
	constructor(info, moneda) {
		this.info = info;
		this.moneda = moneda;
		this.getData().then(this.onDataReady.bind(this));
		this.render();
	}

	render() {
		$('#sidebar_right_errors').hide();
		$('#rightsidebarContainer').html($('<div>', {class: 'LoaderContainer'}).append(new Loader()));
		this.setTitle();
		this.table = new cuentas_CuotasTable(this.info.cuotas, this.moneda)

		this.form = $('<form>', {id: "form_cuotas"}).append([
	        $('<div>', {class: "detalleCuotas"}).html(this.table.get()),
	        $('<div>', {class: "block-header"}).append(
	            $('<h2>', {class: "actionsCuotaTitle"}).html('AGREGAR CUOTA')
	        	), 
	        $('<div>', {class: "input-group"}).append([
	            $('<div>', {class: "form-line"}).append([
	                $('<span>', {class: "input-group-addon", style:'width:unset'}).html(sistema_TipoTransaccion.getCurrencySymbol(this.moneda)),
	                $('<input>', {type: "text", class: "montoCuota form-control", placeholder: "Monto"})
	            ])
	        ]),
	        $('<div>', {class: "form-group"}).append([
	            $('<div>', {class: "form-line"}).append([
	                this.medioPagoSelect = $('<select>', {name: "medio_pago", id: "medio_pago", class: "form-control"}).append(
	                    $('<option>', {hidden: true, disabled: true, selected: true}).html('Elija medio de pago')
	                )
	            ])
	        ]), 
	        this.tipoCambioContainer = $('<div>', {class: "input-group", style: 'display:none'}).append([
	            $('<div>', {class: "form-line"}).append([
	                this.tipoCambioAddOn = $('<span>', {class: "input-group-addon", style:'width:unset'}),
	                $('<input>', {type: "text", class: "montoCambio form-control", placeholder: "Tipo de Cambio"})
	            ])
	        ]),
	        $('<div>', {class: "form-group"}).append([
	            $('<div>', {class: "form-line"}).append([
	                this.oficinaSelect = $('<select>', {name: "oficina", id: "oficina", class: "form-control"}).append(
	                    $('<option>', {hidden: true, disabled: true, selected: true}).html('Elija oficina')
	                )
	            ])
	        ]),
	        $('<div>', {class:"form-group", style: "text-align: right"}).append([
	            this.saveButton = $('<button>', {class: "btn-save-cuota btn btn-success"}).html('Guardar'), ' ',
	            $('<button>', {class: "btnCancelCuota btn btn-danger", id: "sidebar_right_cancelar"}).html('Cancelar')
        	])
		]);

		this.saveButton.on('click', this.onClickSave.bind(this));
	}

	get() {
		return this.form;
	}

	setTitle() {
		$('#sidebar_right_title').html('DETALLE CUOTAS');
	}

	getData() {
    	return $.ajax({
    		url: base_url + '/caja/getformdata',
    		type: 'GET'
    	});	
    }

	onDataReady(response) {
    	this.oficinas = JSON.parse(response[0]);
    	this.mediosPago = this.filterTransactionTypes(JSON.parse(response[1]));
    	this.appendOficinasOptions();
    	this.appendMediosPagoOptions();
		$('.montoCuota').mask("#.##0,00", {reverse: true});
		$('.montoCambio').mask("#.##0,00", {reverse: true});
    	$('#rightsidebarContainer').html(this.form);
    }

    filterTransactionTypes(transactions) {
    	return transactions.filter(function(transaction) {
			return transaction.id != 6 && transaction.id != 7;
    	}, this);
    }

    appendOficinasOptions() {
    	var options = [];
    	this.oficinas.forEach(function(oficina) {
    		var option = $('<option>', {value: oficina.id}).html(oficina.nombre);
    		options.push(option);
    	}, this);
    	this.oficinaSelect.append(options);
        if (this.transaction) {
            this.oficinaSelect.val(this.transaction.oficina_id);
        }
    }

    appendMediosPagoOptions() {
    	var options = [];
    	this.mediosPago.forEach(function(medioPago) {
    		var option = $('<option>', {value: medioPago.id}).html(medioPago.nombre);
    		options.push(option);
    	}, this);
    	this.medioPagoSelect.append(options);
        if (this.transaction) {
            this.medioPagoSelect.val(this.transaction.tipo_transaccion_id);
        }

        this.medioPagoSelect.change(this.onMedioDePagoSelectChange.bind(this));
    }


	onClickSave(event) {
		event.preventDefault();
		var cuota = this.getValue();
		this.saveCuota(cuota);
	}

	getValue() {
		var montoEntregado = $('#form_cuotas .montoCuota').val();
		return {
			monto: stringToFloatDecimals(montoEntregado),
			pasajero_id: this.info.pasajero_id,
			reserva_id: this.info.reserva_id,
			reserva_info_id: this.info.id,
			viaje_id: this.info.viaje_id,
			medio_pago_id: $('#form_cuotas #medio_pago').val(),
			monedaViaje: this.moneda,
			monto_cambio: this.isPayingWithDifferentCurrency() ? stringToFloatDecimals($('#form_cuotas .montoCambio').val()) : null,
			oficina_id: $('#form_cuotas #oficina').val()
		};
	}

	saveCuota(data) {
		$.ajax({
			url: base_url+'/cuentas/saveCuota',
			type: 'POST',
			data: {
				_token: $('meta[name="csrf-token"]').attr('content'),
				data: data
			},
			success: this.onSave.bind(this)
		})
	}

	onSave(response) {
		var pasajero = $('#form_cuotas').data('pasajero');
		if(response.errors) {
			showErrors('#sidebar_right_errors', response.errors);
		} else {
			var cuota = $.parseJSON(response);
			this.info.cuotas.push(cuota);
			this.form.trigger('savecuota', cuota);

			swal({
				title: 'Hecho',
				text: 'Cuota guardada con exito.',
				icon:'success'
			});
		}
	}

	onMedioDePagoSelectChange(event) {
		event.preventDefault();
		if (this.isPayingWithDifferentCurrency()) {
			this.tipoCambioContainer.show();
			this.tipoCambioAddOn.html(sistema_TipoTransaccion.getCurrencySymbol(this.medioPagoSelect.val()))
		} else {
			this.tipoCambioContainer.hide();
		}
	}

	isPayingWithDifferentCurrency() {
		return (
			this.moneda == sistema_TipoTransaccion.DOLARES && this.medioPagoSelect.val() != sistema_TipoTransaccion.DOLARES
		) || (
			this.moneda == sistema_TipoTransaccion.EUROS && this.medioPagoSelect.val() != sistema_TipoTransaccion.EUROS
		);
	}
}