class cuentas_Row {
	constructor(info, reserva, viaje) {
		this.info = info;
		this.reserva = reserva;
		this.viaje = viaje;
		this.converterMethod = sistema_TipoTransaccion.getCurrencyMethod(this.viaje.moneda);
		this.initProps();
		this.calculateMontos();
		this.render();
	}

	render() {
		this.row = $('<tr>', {class: 'pasajero' + this.info.pasajero.id}).append([
			$('<td>').html(this.info.reserva_id),
			this.userInvolved = $('<td>').html(this.reserva.user.lastName + ', ' + this.reserva.user.name),
			$('<td>', {class: "colAsiento"}).html('<span class="busSeat cuentasRow">' + this.info.nro_asiento + '</span>'),
			$('<td>').html(this.info.pasajero.apellido + ', ' + this.info.pasajero.nombre),
			$('<td>').html(this.info.pasajero.dni),
			$('<td>').html(moment(this.info.pasajero.fecha_nacimiento).format('DD/MM/YYYY')),
			$('<td>').html(this.info.pasajero.localidad),
			$('<td>').html(this.info.pasajero.telefono),
			$('<td>').append([
				this.subeEn = $('<span>').html(this.info.sube_en ? this.info.sube_en : this.info.pasajero.localidad),
			]),
			this.roomColumn = this.viaje.noches > 0 ? $('<td>').html(this.reserva.habitacion ? this.reserva.habitacion.nombre : null) : null,
			this.infoExtra = $('<td>').html(this.info.info),
			this.tarifa = $('<td>', {style: "text-align: right"}).html(this.converterMethod(this.getTarifa())),
			this.columnEntregado = $('<td>').html(this.converterMethod(this.entregado)),
			this.columnSaldo = $('<td>').html(this.converterMethod(this.saldo)),
			$('<td>').html(this.createMenu())
		]);

		this.editButton.on('click', this.onClickEdit.bind(this));
		this.detallesButton.on('click', this.onClickDetallesCuotas.bind(this));
		this.cancelReservaButton.on('click', this.onAtemptCancelReserva.bind(this));
	}

	get() {
		return this.row;
	}

	getTarifa() {
		return this.info.precio_opcional != null && this.info.precio_opcional != undefined ? this.info.precio_opcional : this.viaje.precio
	}

	initProps() {
		this.entregado = 0;
		this.saldo = this.getTarifa();
	}

	createMenu() {
		return $('<div>', {class: "dropdown", style: "display: inline"}).append([
            $('<button>', {class: "btn btn-default dropdown-toggle", type: "button", 'data-toggle': "dropdown", 'aria-haspopup': "true", 'aria-expanded': "true"}).html(
                '<i class="fas fa-bars"></i><span class="caret"></span>'
            ),
            $('<ul>', {class: "dropdown-menu dropdown-menu-right", 'aria-labelledby': "dropdownMenu1"}).append([
                this.detallesButton = $('<li>').html('<a href="#" class="dropdown-item js-right-sidebar"><i class="fas fa-file-alt"></i> Ver Detalle</a>'),
                this.editButton = $('<li>').html('<a href="#" class="dropdown-item js-right-sidebar"><i class="fas fa-edit"></i> Editar</a>'),
                this.cancelReservaButton = $('<li>').html('<a href="#" class="dropdown-item"><i class="fas fa-ban"></i> Cancelar Reserva</a>')
            ])
		]);
	}

	calculateMontos() {
		this.info.cuotas.forEach(function(cuota) {
			this.entregado += cuota.transaction.monto;
			this.saldo -= cuota.transaction.monto;
		}, this);
	}

	getEntregado() {
		return this.entregado;
	}

	onClickDetallesCuotas() {
		event.preventDefault();
		var form = new cuentas_FormCuotas(this.info, this.viaje.moneda);
		form.get().on('savecuota', this.onSaveCuota.bind(this));
	}

	onSaveCuota(event, cuota) {
		this.updateMontos();
		this.row.trigger('savecuota', cuota);
	}

	updateMontos() {
		this.initProps();
		this.calculateMontos();
		this.columnEntregado.html(this.converterMethod(this.entregado));
		this.columnSaldo.html(this.converterMethod(this.saldo));
	}

	onClickEdit() {
		event.preventDefault();
		var form = new cuentas_FormReserva(this.info, this.viaje, this.reserva);
		form.get().on('updatereserva', this.onUpdateReserva.bind(this));
	}

	onUpdateReserva(event) {
		this.userInvolved.html(this.reserva.user.lastName + ', ' + this.reserva.user.name);
		this.subeEn.html(this.info.sube_en ? this.info.sube_en : this.info.pasajero.localidad);
		this.infoExtra.html(this.info.info ? this.info.info : '');
		this.tarifa.html(this.converterMethod(this.getTarifa()));
		if (this.roomColumn && this.reserva.habitacion) {
			this.roomColumn.html(this.reserva.habitacion.nombre)
		}
		this.updateMontos();
		this.row.trigger('updatereserva', this.info.pasajero_id);
	}

	onAtemptCancelReserva(event) {
		event.preventDefault();
		swal({
            title: 'Cancelar Reserva',
            text: 'Esta seguro que desea cancelar esta reserva?',
            icon: 'warning',
            buttons: ['Cancelar', 'Aceptar']
        }).then((value) => {
        	if(value) {
				this.cancelReserva();
        	}
        })
	}

	cancelReserva() {
		var data = {
			id: this.info.id,
			reserva_id: this.info.reserva_id
		};
		$.ajax({
			url: base_url + '/cuentas/cancelReserva',
			type: 'POST',
			data: {
				_token: token,
				data: data
			},
			success : this.onCancelReserva.bind(this)
		})
	}

	onCancelReserva() {
		this.row.trigger('cancelreserva', [this.info]);
	}
}