class cuentas_Table {
	constructor(viaje) {
		this.viaje = viaje;
		this.converterMethod = sistema_TipoTransaccion.getCurrencyMethod(this.viaje.moneda);
		this.getReservasViaje(this.viaje).then(this.onReservasReady.bind(this));
		this.initProps();
		return this.render();
	}

	render() {
		this.table = $('<table>', {class: "table table-bordered tabla-circuitos tabla-cuentas"}).append([
			$('<thead>').append([
				$('<tr>').append([
					$('<th>').html('Codigo'),
					$('<th>').html('Vendedor'),
					$('<th>').html('Nro Asiento'),
					$('<th>').html('Pasajero'),
					$('<th>').html('DNI'),
					$('<th>').html('Nacimiento'),
					$('<th>').html('Localidad'),
					$('<th>').html('Telefono'),
					$('<th>').html('Punto Ascenso'),
					!this.isCircuitoRegional() ? $('<th>').html('Tipo Habitacion') : null,
					$('<th>').html('Info Extra'),
					$('<th>').html('Tarifa'),
					$('<th>').html('Entregado'),
					$('<th>').html('Saldo'),
					$('<th>').html('')
				])
			]),
			this.tableBody = $('<tbody>').append(this.createLoader()),
			$('<tfoot>').append([
				$('<tr>').append([
					$('<th>', {colspan: !this.isCircuitoRegional() ? 11 : 10, class: "tdNoDisplay"}),
					$('<th>', {style: 'font-weight: bold'}).html('TOTAL A COBRAR <span class="totalACobrar"></span>'),
					$('<th>', {style: 'font-weight: bold'}).html('TOTAL COBRADO <span class="totalCobrado"></span>'),
					$('<th>', {style: 'font-weight: bold'}).html('TOTAL SALDO <span class="saldoTotal"></span>'),
					$('<th>', {class: "tdNoDisplay"}).html('')
				])
			])
		])
		
		this.table.on('savecuota', this.onSaveCuota.bind(this));
		this.table.on('updatereserva', this.onUpdateReserva.bind(this));
		this.table.on('cancelreserva', this.onCancelReserva.bind(this));
		
		return this.table;
	}

	initProps() {
		this.totalACobrar = 0;
		this.totalEntregado = 0;
		this.totalSaldo = 0;
	}

	initDatatables() {
		var targets = [5,6,7, 8, 9, 10, 11, 12, 13];
		if (this.viaje.noches > 0) {
			targets.push(14)
		}
		this.table.DataTable({
	        pageLength: 25,
	        bPaginate: false,
	    	bLengthChange: false,
	    	bInfo: false,
	    	order: [2, 'asc'],
	        language: {
	            url: base_url+'/../js/vendors/dataTables/spanish.json'
	        },
	        columnDefs: [
	        	{orderable: false, targets: targets}
	        ]
	    });
	}

	getReservasViaje() {
		return $.ajax({
			url: base_url + '/cuentas/'+ this.viaje.id +'/getReservasViaje',
			type: 'GET',
			data: {viajeId: this.viaje.id}
		});
	}

	onReservasReady(response) {
		this.reservas = response;
		this.createTableContent();
		this.initDatatables();
	}

	isCircuitoRegional() {
		return this.viaje.noches == 0;
	}

	createLoader() {
		return this.appendUniqueColumn(new Loader);
	}

	appendUniqueColumn(content) {
		return $('<tr>').append(
			$('<td>', {
				'class': 'uniqueColumn',
				'colspan': 15
			}).append(content)
		);
	}

	createTableContent() {
		if (this.reservas.length == 0) {
			this.tableBody.html(
				this.appendUniqueColumn('No hay reservas realizadas para este viaje')
			);
			return;
		}
		var rows = [];
		this.reservas.forEach(function(reserva) {
			reserva.info.forEach(function(info) {
				var row = new cuentas_Row(info, reserva, this.viaje)
				rows.push(row.get());
				this.totalACobrar += info.precio_opcional ? info.precio_opcional : this.viaje.precio;
				this.totalEntregado += parseFloat(row.getEntregado());
			}, this)
		}, this);
		this.tableBody.html(rows);

		this.totalSaldo = this.totalACobrar - this.totalEntregado;

		$('.totalACobrar').html(this.converterMethod(this.totalACobrar));
		$('.totalCobrado').html(this.converterMethod(this.totalEntregado));
		$('.saldoTotal').html(this.converterMethod(this.totalSaldo));
	}

	onSaveCuota(event, cuota) {
		this.totalEntregado += cuota.transaction.monto;
		this.totalSaldo -= cuota.transaction.monto;

		$('.totalCobrado').html(this.converterMethod(this.totalEntregado));
		$('.saldoTotal').html(this.converterMethod(this.totalSaldo));
	}

	onCancelReserva(event, info) {
		this.updateMontosAfterCancelReserva(info);
		this.table.DataTable()
			.row($('.pasajero' + info.pasajero_id))
			.remove()
			.draw();

		swal({title: 'Hecho', text: 'Reserva cancelada con exito', icon: 'success'});
	}

	onUpdateReserva(event, pasajeroId) {
		var tr = $('.pasajero' + pasajeroId);
		this.table.DataTable().row(tr).invalidate().draw();
		
		this.totalACobrar = 0;
		this.totalSaldo = 0;
		this.reservas.forEach(function(reserva) {
			reserva.info.forEach(function(info) {
				this.totalACobrar += info.precio_opcional ? info.precio_opcional : this.viaje.precio;
			}, this)
		}, this);
		
		this.totalSaldo = this.totalACobrar - this.totalEntregado;

		$('.totalACobrar').html(this.converterMethod(this.totalACobrar));
		$('.saldoTotal').html(this.converterMethod(this.totalSaldo));
	}

	updateMontosAfterCancelReserva(info) {
		var montoTotal = 0;
		info.cuotas.forEach(function(cuota) {
			montoTotal += cuota.transaction.monto;
		}, this);
		
		this.totalACobrar -= info.precio_opcional ? info.precio_opcional : this.viaje.precio;
		this.totalEntregado -= montoTotal;
		this.totalSaldo = this.totalACobrar - this.totalEntregado;

		$('.totalACobrar').html(this.converterMethod(this.totalACobrar));
		$('.totalCobrado').html(this.converterMethod(this.totalEntregado));
		$('.saldoTotal').html(this.converterMethod(this.totalSaldo));
	}
}