class reserva_Form {
	constructor(viaje) {
		this.viaje = viaje;
		this.selectedSeats = [];
		this.roomSelectorField = null;
		this.getReservaData().then(this.onDataReady.bind(this));
		this.render();
	}

	render() {
		this.form = $('<form>', {id: "form_reserva"});
	}

	get() {
		return this.form;
	}

	getReservaData(isUserAdmin) {
		var isUserAdmin = userSession.role == 2;
		return $.ajax({
			url: base_url+'/reservar/' + this.viaje.id + '/getFormData',
			type: 'GET',
			data: {
				isUserAdmin: isUserAdmin
			}
		});
	}
	
	onDataReady(response) {
		this.pasajerosDisponibles = response['pasajeros'];
		if ('users' in response) {
			this.users = response['users'];
		}
		this.roomTypes = Array.from(JSON.parse(response['roomTypes']));
		this.reservas = Array.from(JSON.parse(response['reservas']));
		
		if (!this.isCircuitoRegional() && this.areRoomsAvailable()) {
			this.roomSelector = new room_Selector(this.roomTypes, this.reservas);
			this.roomSelectorField = $('<div>', {class: "form-group"}).append([
				this.roomSelector.get()
			]);
		}

		this.form.append([
    		$('<div>', {id: "reserva_errors", class: "alert alert-danger", style: "display:none"}),
	        $('<ul>'),
	        this.rowsContainer = $('<div>', {class: 'rowsContainer'}),
	        this.extraControlsContainer = $('<div>').append([
	        	$('<hr>', {class: 'separator', style: 'border-color: grey'}),
				$('<div>', {class: 'userSelector'}).append([
				$('<div>', {id: 'secondSectionError', class:"alert alert-danger", style:"display:none"}).html($('<ul>')),
					this.roomSelectorField,
					this.createUserSelectorIfNeeded()
				])
	        ]),
			$('<div>', {style: "text-align: right"}).append(
				this.reservarButton = $('<button>', {class: "btn btn-lg bg-teal"}).html('<strong>Reservar</strong>')
			)
		]);

		this.userSelector.selectpicker();
		this.reservarButton.on('click', this.onClickReservar.bind(this));

		this.form.trigger('ready');
	}

	renderSeatForm(nroAsiento) {
		var nroTitulo = nroAsiento < 10 ? '0' + nroAsiento : nroAsiento;
		var select = this.createPasajerosSelect(nroAsiento);

		var selectedSeatForm = $('<div>', {class: 'reserva-seat'}).append([
			$('<div>', {class: 'seat seat-' + nroAsiento}).append([
				$('<h4>').html('ASIENTO ' + nroTitulo),
				$('<div>', {
					id: 'reserva_errors_' + nroAsiento,
					class:"alert alert-danger alert-reserva",
					style:"display:none"
				}).html($('<ul>'))
				,
				$('<div>', {class: "form row"}).append([
					$('<div>', {class: "colSelectPassenger col-lg-10 selectSearcher"}).append([
						$('<div>', {class: "form-group"}).html(select)
					]),
					$('<div>', {class: "colNewPassenger col-lg-2"}).append([
						$('<div>', {class: "form-group"}).append([this.getCrearPasajeroBtn()])	
					])
				]),
				$('<div>', {class: "form-group"}).append([
					$('<div>', {class: "input-group"}).append([
						$('<div>', {class: "form-line"}).append([
							$('<span>', {class: "input-group-addon"}).html('$'),
							$('<input>', {
								type: "text",
								id: 'precioOpcional' + nroAsiento,
								class:"form-control precio_opcional",
								placeholder: "Precio opcional"
							})
						])
					])
				])
			]).data('seat', nroAsiento)
		]);

		this.rowsContainer.append(selectedSeatForm);

		$('#precioOpcional' + nroAsiento).mask("#.##0,00", {reverse: true});
		select.selectpicker();
		
		$(document).on('change', '#select-passenger-' + nroAsiento, this.onSelectChange.bind(this));
	}

	addSelectedSeat(nroAsiento) {
		this.renderSeatForm(nroAsiento);
		this.selectedSeats.push(this.createSeat(nroAsiento));
	}

	createSeat(nro_asiento) {
		return {
			viajeId: this.viaje.id,
			pasajero_id: null,
			nro_asiento: nro_asiento,
			precio_opcional: null
		}
	}

	onSelectChange(event) {
		event.preventDefault();
		var selects = $('select.select-passenger');
		var currentSelect = $(event.currentTarget);
		var selectedValue = currentSelect.val();
	 	selects.each(function() {
	 		if($(this).attr('id') !== currentSelect.attr('id')) {
	 			if($(this).val() == selectedValue) {
	 				swal({
	 					title: 'Error',
	 					text: 'Este pasajero ya fue seleccionado.',
	 					icon: 'error'
	 				})
	 				currentSelect.selectpicker('val', null);
	 			}
	 		}
	 	});
	 	this.pasajerosDisponibles.forEach(function(pasajero) {
	 		if (pasajero.id == selectedValue) {
				currentSelect.data('pasajero', pasajero);
	 		}
	 	}, this);
	}

	createPasajerosSelect(nroAsiento){
		var select = $('<select>', {
			id: 'select-passenger-' + nroAsiento,
			class: "form-control select-passenger",
			'data-live-search': true,
			title: "Seleccione un pasajero"
		});
		var options = [];
		this.pasajerosDisponibles.forEach(function(pasajero, index) {
		    options.push(
		    	$('<option>', {value: pasajero.id}).html(
		    		pasajero.apellido+', '+pasajero.nombre+' | DNI: '+pasajero.dni+' | '+pasajero.localidad
		    	)
		    );
		});
		select.append(options);
		return select;
	}

	isCircuitoRegional() {
		return this.viaje.noches == 0;
	}

	isSelectionEmpty() {
		return isEmpty(this.selectedSeats);
	}

	removeSelectedSeat(seat) {
		$('.seat-' + seat).remove();
		this.selectedSeats.forEach(function(selectedSeat, index) {
			if (selectedSeat.nro_asiento == seat) {
				this.selectedSeats.splice(index, 1);
			}
		}, this);
	}

	onClickReservar(event) {
		event.preventDefault();
		this.selectedSeats.forEach(function(seat) {
			seat.pasajero_id = $('#select-passenger-' + seat.nro_asiento).val();
			seat.pasajero = $('#select-passenger-' + seat.nro_asiento).data('pasajero');
			seat.precio_opcional = stringToFloatDecimals($('.seat-' + seat.nro_asiento + ' .precio_opcional').val());
		}, this);
		this.userInvolved = $('#userInvolved').val();
		this.roomSelected = $('#roomSelect').val();

		this.validateData();
	}

	areRoomsAvailable() {
		return this.roomTypes.length > 0;
	}

	validateData() {
		$.ajax({
			url: base_url+'/reservar/'+this.viaje.id+'/validar',
			type: 'POST',
			data: {
				_token: token,
				cantNoches: this.viaje.noches,
				areRoomsAvailable: this.areRoomsAvailable() ? 1 : 0,
				user_id: this.userInvolved,
				selectedSeats: this.selectedSeats,
				roomSelected: this.roomSelected
			},
			success: this.onValidateReserva.bind(this)
		});
	}

	onValidateReserva(data) {
		if (data.errors) {
			showErrors('#secondSectionError', data.errors);
			return;
		} else {
        	$('#secondSectionError').slideUp(250, function(){
				$(this).hide();
			});
		}

		if (!this.isCircuitoRegional() && this.areRoomsAvailable() && !this.roomSelector.validateSelectedRoom(this.selectedSeats.length)) {
			return;
		}

		if (data.messages && data.messages.length > 0) {
			$.each(data.messages, function(index, seatMessages) {
            	showErrors('#reserva_errors_' + seatMessages.key, seatMessages.value.original.errors)
			});
			return;
        } else {
        	$('.alert-reserva').each(function() {
        		$(this).slideUp(250, function(){
    				$(this).hide();
				})
        	});
        }
		
    	swal({
    		title: 'Confirmar',
    		text: 'Esta seguro que desea confirmar esta reserva?',
    		icon: 'warning',
    		buttons: ['Cancelar', 'Aceptar'],
    	}).then(this.onConfirmReserva.bind(this))
	}

	onConfirmReserva(value) {
		if(!value) {
			return;
		}
		$.ajax({
			url: base_url+'/reservar/'+this.viaje.id+'/confirmar',
			type: 'POST',
			data: {
				_token: token,
				user_id: this.userInvolved,
				selectedSeats: this.selectedSeats,
				tipo_habitacion_id: this.roomSelected
			},
			success: this.goToReservaInfo.bind(this)
		})
	}

	goToReservaInfo(data) {
		if (data) {
			window.location.href = base_url + '/reservar/confirmacion/' + this.viaje.id +
				'?seats=' + JSON.stringify(this.selectedSeats) + '&roomSelected=' + this.roomSelected + '&reserva=' + JSON.stringify(data);
		}
	}

	createUserSelectorIfNeeded() {
		if (userSession.role != 2) {
			return;
		}
		this.userSelector = $('<select>', {class: 'form-control', id: 'userInvolved', 'data-live-search':"true", title:"Seleccione un usuario"}).append([
			$('<option>', {disabled: true, hidden: true, selected: true}).html('Seleccione el usuario que realizo la reserva')
		]);
		this.users.forEach(function(user) {
			var option = $('<option>', {value: user.id}).html(user.lastName + ', ' + user.name);
			this.userSelector.append(option);
		}, this)
		return $('<div>', {class: 'form-group selectSearcher'}).append([
			this.userSelector
		]);
	}

	getCrearPasajeroBtn() {
		var button = $('<button>', {
			class: "btn btn-block btn-primary js-right-sidebar",
			'data-close': "true"
		}).html('<i class="fas fa-plus"></i> Crear')

		button.on('click', this.onClickCrearPasajero.bind(this));
		return button;
	}

	onClickCrearPasajero(event) {
		event.preventDefault();
		var selectedSeat = $(event.currentTarget).parents('.seat').data('seat');
		var form = new pasajero_Form();
		form.get().on('savecompleted', this.onPasajeroSaveCompleted.bind(this, selectedSeat))
	}

	onPasajeroSaveCompleted(selectedSeat, event, pasajero) {
		var optionText = pasajero.apellido + ', ' + pasajero.nombre + ' | DNI: ' + pasajero.dni + ' | ' + pasajero.localidad;
		var newOption = $('<option>', {value: pasajero.id}).html(optionText);
        var select = $('#select-passenger-' + selectedSeat);
        select.append(newOption);
        select.selectpicker('refresh');
        select.data('pasajero', pasajero);
        select.selectpicker('val', pasajero.id);
	}
}