class room_AvailabilityCalculator {
	constructor(roomTypes, reservas, shouldCalculateShareable) {
		this.roomTypes = roomTypes;
		this.reservas = reservas;
		this.shouldCalculateShareable = shouldCalculateShareable;
	}

	calculate() {
		this.roomsAvailability = {};
		this.roomTypes.forEach(function(roomType) {
			this.roomsAvailability[roomType.tipo_habitacion_id] = {
				cantidad: roomType.cantidad,
				cantidadCamas: roomType.type.cantidad_camas,
				nombre: roomType.type.nombre
			};
		}, this);
		this.reservas.forEach(function(reserva) {
			var roomType = reserva.tipo_habitacion_id;
			if (!roomType || !this.roomsAvailability.hasOwnProperty(roomType)) {
				return;
			}
			this.roomsAvailability[roomType].cantidad--;
			if (this.shouldCalculateShareable) {
				this.calculateShareableRoomAvailability(reserva);
			}
		}, this);
		return this.roomsAvailability;
	}

	calculateShareableRoomAvailability(reserva) {
		if (reserva.tipo_habitacion_id == 2) {
			this.roomsAvailability[8].cantidad -= 2;
		} else if (reserva.tipo_habitacion_id == 4) {
			this.roomsAvailability[9].cantidad -= 3;
		}

		if (this.isShareableRoom(reserva.tipo_habitacion_id)) {
			var roomTypeId = reserva.tipo_habitacion_id == 8 ? 2 : 4;
			if (this.isDoubleRoom(roomTypeId)) {
				this.sharingDouble ++;
				this.calculateShareableDoubleRoom(reserva);
			} else {
				this.sharingTriple ++;
				this.calculateShareableTripleRoom(reserva);
			}
		}
	}

	isShareableRoom(roomTypeId) {
		return room_AvailabilityCalculator.SHAREABLE_ROOMS.includes(roomTypeId);
	}

	isDoubleRoom(roomTypeId) {
		return roomTypeId == 2;
	}

	calculateShareableDoubleRoom(reserva) {
		if (this.sharingDouble % 2 != 0) {
			this.roomsAvailability[2].cantidad--;
		}
	}

	calculateShareableTripleRoom(reserva) {
		if (this.sharingTriple % 3 != 0 && this.sharingTriple % 3 <= 1) {
			this.roomsAvailability[4].cantidad--;
		}
	}
}

room_AvailabilityCalculator.SHAREABLE_ROOMS = [8, 9];